# Copyright (C) 2006-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk


class TestSearchDialog(gaupol.TestCase):

    def run__dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def run__show_regex_error_dialog_pattern(self):
        self.dialog._show_regex_error_dialog_pattern("test")

    def run__show_regex_error_dialog_replacement(self):
        self.dialog._show_regex_error_dialog_replacement("test")

    def setup_method(self, method):
        gaupol.conf.search.max_history = 2
        gaupol.conf.editor.use_custom_font = True
        gaupol.conf.editor.custom_font = "sans"
        self.application = self.new_application()
        self.dialog = gaupol.SearchDialog(self.application)
        self.dialog.show()

    def test__on_all_radio_toggled(self):
        self.dialog._current_radio.set_active(True)
        self.dialog._all_radio.set_active(True)
        self.dialog._current_radio.set_active(True)

    def test__on_application_page_changed(self):
        # Ensure that editing obsolete data is not possible.
        # http://bugzilla.gnome.org/show_bug.cgi?id=572676
        self.dialog._pattern_entry.set_text("a")
        self.dialog.next()
        page = self.application.get_current_page()
        page.project.remove_subtitles((self.dialog._match_row,))
        assert not self.dialog._text_view.props.sensitive
        assert not self.dialog._replace_button.props.sensitive

    def test__on_ignore_case_check_toggled(self):
        self.dialog._ignore_case_check.set_active(True)
        self.dialog._ignore_case_check.set_active(False)
        self.dialog._ignore_case_check.set_active(True)

    def test__on_next_button_clicked(self):
        self.dialog._pattern_entry.set_text("a")
        self.dialog._next_button.emit("clicked")

    def test__on_main_check_toggled(self):
        self.dialog._main_check.set_active(True)
        self.dialog._tran_check.set_active(True)
        self.dialog._main_check.set_active(True)

    def test__on_pattern_entry_changed(self):
        self.dialog._pattern_entry.set_text("a")
        self.dialog._pattern_entry.set_text("")

    def test__on_previous_button_clicked(self):
        self.dialog._pattern_entry.set_text("a")
        self.dialog._previous_button.emit("clicked")

    def test__on_regex_check_toggled(self):
        self.dialog._regex_check.set_active(True)
        self.dialog._regex_check.set_active(False)
        self.dialog._regex_check.set_active(True)

    def test__on_replace_all_button_clicked(self):
        self.dialog._pattern_entry.set_text("e")
        self.dialog._replacement_entry.set_text("x")
        self.dialog._replace_all_button.emit("clicked")

    def test__on_replace_button_clicked(self):
        self.dialog._pattern_entry.set_text("a")
        self.dialog._replacement_entry.set_text("x")
        self.dialog._next_button.emit("clicked")
        self.dialog._replace_button.emit("clicked")

    def test__on_response(self):
        self.dialog.response(gtk.RESPONSE_HELP)

    def test__on_show(self):
        self.dialog.show()
        self.dialog.hide()
        self.dialog.show()

    def test__on_text_view_focus_out_event(self):
        self.dialog._pattern_entry.set_text("a")
        self.dialog._next_button.emit("clicked")
        gaupol.util.iterate_main()
        self.dialog._next_button.emit("clicked")
        gaupol.util.iterate_main()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__set_pattern__re_error(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.dialog._regex_check.set_active(True)
        self.dialog._pattern_entry.set_text("*")
        self.dialog.next()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_regex_error_dialog_pattern(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.dialog._show_regex_error_dialog_pattern("test")

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__show_regex_error_dialog_replacement(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.dialog._show_regex_error_dialog_replacement("test")

    def test_next(self):
        self.dialog._regex_check.set_active(True)
        for char in "aeiouy":
            self.dialog._pattern_entry.set_text(char)
            self.dialog.next()
            self.dialog.next()

    def test_next__not_found(self):
        self.dialog._regex_check.set_active(True)
        self.dialog._pattern_entry.set_text("xxx")
        self.dialog.next()
        self.dialog._all_radio.set_active(True)
        self.application.open_main(self.new_subrip_file())
        self.dialog.next()

    def test_previous(self):
        self.dialog._regex_check.set_active(True)
        for char in "aeiouy":
            self.dialog._pattern_entry.set_text(char)
            self.dialog.previous()
            self.dialog.previous()

    def test_previous__not_found(self):
        self.dialog._regex_check.set_active(True)
        self.dialog._pattern_entry.set_text("xxx")
        self.dialog.previous()
        self.dialog._all_radio.set_active(True)
        self.application.open_main(self.new_subrip_file())
        self.dialog.previous()

    def test_replace(self):
        self.dialog._regex_check.set_active(True)
        for char in "aeiouy":
            self.dialog._pattern_entry.set_text(char)
            self.dialog._replacement_entry.set_text(char)
            self.dialog.next()
            self.dialog.replace()
            self.dialog.replace()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_replace__re_error(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.dialog._regex_check.set_active(True)
        self.dialog._pattern_entry.set_text(" ")
        self.dialog._replacement_entry.set_text("\\1")
        self.dialog.next()
        self.dialog.replace()

    def test_replace_all(self):
        self.dialog._regex_check.set_active(True)
        self.dialog._pattern_entry.set_text("^")
        self.dialog._replacement_entry.set_text("-")
        self.dialog.replace_all()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_replace_all__re_error(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.dialog._regex_check.set_active(True)
        self.dialog._pattern_entry.set_text("^")
        self.dialog._replacement_entry.set_text("\\1")
        self.dialog.replace_all()



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestModule(aeidon.TestCase):

    def test_attributes(self):
        for framerate in aeidon.framerates:
            assert hasattr(framerate, "label")
            assert hasattr(framerate, "mpsub")
            assert hasattr(framerate, "value")

    def test_items(self):
        assert hasattr(aeidon.framerates, "FPS_23_976")
        assert hasattr(aeidon.framerates, "FPS_24_000")
        assert hasattr(aeidon.framerates, "FPS_25_000")
        assert hasattr(aeidon.framerates, "FPS_29_970")



# Copyright (C) 2005-2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Managing revertable actions.

To hook a method up with the undo/redo system, the following need to be done:

(1) The last argument to the method must be a keyword argument `register` with
    a default value of -1. This argument indicates which of doing, undoing or
    redoing is in process.

(2) At the end of the method, :meth:`register_action` needs to be called with
    an instance of :class:`aeidon.RevertableAction`. Calling this method takes
    care of the management of undo and redo stacks.

(3) The method should be marked with the :func:`aeidon.revertable` decorator.
    This decorator takes care of emitting an action signal once the method has
    been run, cuts the undo and redo stacks if needed and defaults the
    `register` keyword argument to :attr:`aeidon.registers.DO`.

Each method marked as revertable should match exactly one action in the undo
and redo stacks. Hence, if a method calls other revertable methods, the
resulting action needs to be grouped as one using :meth:`group_actions`.

If a revertable method needs to be performed without the possibility of
reverting, the `register` keyword argument should be given a value of ``None``.
This way it will not be in any way processed by the undo/redo system.
"""

import aeidon


class RegisterAgent(aeidon.Delegate):

    """Managing revertable actions.

    :ivar _do_description: Original description of the action
    """

    # pylint: disable=E1101,W0201

    __metaclass__ = aeidon.Contractual

    def __init__(self, master):
        """Initialize a :class:`RegisterAgent` object."""
        aeidon.Delegate.__init__(self, master)
        self._do_description = None
        aeidon.util.connect(self, self, "notify::undo_limit")

    def _break_action_group_require(self, stack, index=0):
        assert 0 <= index < len(stack)

    def _break_action_group(self, stack, index=0):
        """Break the action group in `stack` into individual actions.

        Return the amount of actions broken into.
        """
        action_group = stack.pop(index)
        for action in reversed(action_group.actions):
            stack.insert(index, action)
        return len(action_group.actions)

    def _get_destination_stack(self, register):
        """Return the stack where the registered action will be placed."""
        if register.shift == 1:
            return self.undoables
        if register.shift == -1:
            return self.redoables
        raise ValueError("Invalid register: %s" % repr(register))

    def _get_source_stack(self, register):
        """Return the stack where the action to register is taken from."""
        if register.shift == 1:
            return self.redoables
        if register.shift == -1:
            return self.undoables
        raise ValueError("Invalid register: %s" % repr(register))

    def _on_notify_undo_limit(self, *args):
        """Cut reversion stacks if limit set."""
        if self.undo_limit is not None:
            self.cut_reversion_stacks()

    def _revert_multiple_require(self, count, register):
        stack = self._get_source_stack(register)
        assert len(stack) >= count

    def _revert_multiple(self, count, register):
        """Revert multiple actions."""
        self.block(register.signal)
        stack = self._get_source_stack(register)
        for i in range(count):
            part_count = 1
            if isinstance(stack[0], aeidon.RevertableActionGroup):
                description = stack[0].description
                part_count = self._break_action_group(stack)
            for j in range(part_count):
                self._do_description = stack[0].description
                stack.pop(0).revert()
            if part_count > 1:
                self.group_actions(register, part_count, description)
        self.unblock(register.signal)
        self.cut_reversion_stacks()
        self.emit_action_signal(register)

    def _shift_changed_value(self, action, shift):
        """Shift the values of changed attributes."""
        if aeidon.documents.MAIN in action.docs:
            self.main_changed += shift
        if tuple(action.docs) == (aeidon.documents.TRAN,):
            # Make translation document active.
            if self.tran_changed is None:
                self.tran_changed = 0
        if aeidon.documents.TRAN in action.docs:
            if self.tran_changed is not None:
                self.tran_changed += shift

    @aeidon.deco.export
    def can_redo(self):
        """Return ``True`` if one or more actions can be redone."""
        return bool(self.redoables)

    @aeidon.deco.export
    def can_undo(self):
        """Return ``True`` if one or more actions can be undone."""
        return bool(self.undoables)

    @aeidon.deco.export
    def cut_reversion_stacks(self):
        """Cut undo and redo stacks to their maximum lengths."""
        if self.undo_limit is not None:
            del self.redoables[self.undo_limit:]
            del self.undoables[self.undo_limit:]

    def emit_action_signal_require(self, register):
        if  register is not None:
            assert self._get_destination_stack(register)

    @aeidon.deco.export
    def emit_action_signal(self, register):
        """Emit an action signal for the most recent registered action."""
        if register is not None:
            self.emit(register.signal,
                      self._get_destination_stack(register)[0])

    def group_actions_require(self, register, count, description):
        if register is not None:
            assert len(self._get_destination_stack(register)) >= count

    def group_actions_ensure(self, value, register, count, description):
        if register is not None:
            assert isinstance(self._get_destination_stack(register)[0],
                              aeidon.RevertableActionGroup)

    @aeidon.deco.export
    def group_actions(self, register, count, description):
        """Group the registered actions as one item in the stack."""
        if register is None: return
        actions = []
        stack = self._get_destination_stack(register)
        for i in range(count):
            actions.append(stack.pop(0))
        action_group = aeidon.RevertableActionGroup()
        action_group.actions = actions
        action_group.description = description
        stack.insert(0, action_group)

    def redo_require(self, count=1):
        assert len(self.redoables) >= count

    @aeidon.deco.export
    def redo(self, count=1):
        """Redo `count` amount of actions from the redoable stack."""
        group = aeidon.RevertableActionGroup
        if (count > 1) or isinstance(self.redoables[0], group):
            return self._revert_multiple(count, aeidon.registers.REDO)
        self._do_description = self.redoables[0].description
        self.redoables.pop(0).revert()

    @aeidon.deco.export
    def register_action(self, action):
        """Register `action` as done, undone or redone."""
        if action.register is None: return
        if action.register == aeidon.registers.DO:
            self.undoables.insert(0, action)
            self.redoables = []
        if action.register == aeidon.registers.UNDO:
            self.redoables.insert(0, action)
            action.description = self._do_description
        if action.register == aeidon.registers.REDO:
            self.undoables.insert(0, action)
            action.description = self._do_description
        self._shift_changed_value(action, action.register.shift)

    def set_action_description_require(self, register, description):
        if register is not None:
            assert self._get_destination_stack(register)

    @aeidon.deco.export
    def set_action_description(self, register, description):
        """Set the description of the most recent registered action."""
        if register is None: return
        stack = self._get_destination_stack(register)
        stack[0].description = description

    def undo_require(self, count=1):
        assert len(self.undoables) >= count

    @aeidon.deco.export
    def undo(self, count=1):
        """Undo `count` amount of actions from the undoable stack."""
        group = aeidon.RevertableActionGroup
        if (count > 1) or isinstance(self.undoables[0], group):
            return self._revert_multiple(count, aeidon.registers.UNDO)
        self._do_description = self.undoables[0].description
        self.undoables.pop(0).revert()



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import os
import sys


class TestModule(aeidon.TestCase):

    @aeidon.deco.monkey_patch(sys, "platform")
    def test__xdg_copy_config_files(self):
        sys.platform = "linux2"
        mkdir = aeidon.temp.create_directory
        rmdir = aeidon.temp.remove_directory
        aeidon.paths.OBSOLETE_PROFILE_DIR = mkdir()
        aeidon.paths.CONFIG_HOME_DIR = mkdir()
        aeidon.paths._xdg_copy_config_files()
        rmdir(aeidon.paths.OBSOLETE_PROFILE_DIR)
        rmdir(aeidon.paths.CONFIG_HOME_DIR)

    @aeidon.deco.monkey_patch(sys, "platform")
    def test__xdg_copy_data_files(self):
        sys.platform = "linux2"
        mkdir = aeidon.temp.create_directory
        rmdir = aeidon.temp.remove_directory
        aeidon.paths.OBSOLETE_PROFILE_DIR = mkdir()
        aeidon.paths.DATA_HOME_DIR = mkdir()
        aeidon.paths._xdg_copy_data_files()
        rmdir(aeidon.paths.OBSOLETE_PROFILE_DIR)
        rmdir(aeidon.paths.DATA_HOME_DIR)

    @aeidon.deco.monkey_patch(sys, "platform")
    def test_config_home_dir__win32(self):
        sys.platform = "win32"
        reload(aeidon.paths)
        assert hasattr(aeidon, "CONFIG_HOME_DIR")

    @aeidon.deco.monkey_patch(os, "environ")
    def test_config_home_dir__xdg_environment(self):
        os.environ["XDG_CONFIG_HOME"] = "xdg"
        reload(aeidon.paths)
        assert hasattr(aeidon, "CONFIG_HOME_DIR")

    @aeidon.deco.monkey_patch(os, "environ")
    def test_config_home_dir__xdg_no_environment(self):
        os.environ.clear()
        reload(aeidon.paths)
        assert hasattr(aeidon, "CONFIG_HOME_DIR")

    @aeidon.deco.monkey_patch(sys, "frozen")
    def test_data_dir__py2exe(self):
        sys.frozen = "py2exe"
        reload(aeidon.paths)
        assert hasattr(aeidon, "DATA_DIR")

    def test_data_dir__source(self):
        assert os.path.isdir(aeidon.DATA_DIR)

    @aeidon.deco.monkey_patch(sys, "platform")
    def test_data_home_dir__win32(self):
        sys.platform = "win32"
        reload(aeidon.paths)
        assert hasattr(aeidon, "DATA_HOME_DIR")

    @aeidon.deco.monkey_patch(os, "environ")
    def test_data_home_dir__xdg_environment(self):
        os.environ["XDG_DATA_HOME"] = "xdg"
        reload(aeidon.paths)
        assert hasattr(aeidon, "DATA_HOME_DIR")

    @aeidon.deco.monkey_patch(os, "environ")
    def test_data_home_dir__xdg_no_environment(self):
        os.environ.clear()
        reload(aeidon.paths)
        assert hasattr(aeidon, "DATA_HOME_DIR")

    @aeidon.deco.monkey_patch(sys, "frozen")
    def test_locale_dir__py2exe(self):
        sys.frozen = "py2exe"
        reload(aeidon.paths)
        assert hasattr(aeidon, "LOCALE_DIR")

    def test_locale_dir__source(self):
        assert hasattr(aeidon, "LOCALE_DIR")

    @aeidon.deco.monkey_patch(sys, "platform")
    def test_xdg_copy_if_applicable(self):
        sys.platform = "linux2"
        mkdir = aeidon.temp.create_directory
        rmdir = aeidon.temp.remove_directory
        aeidon.paths.OBSOLETE_PROFILE_DIR = mkdir()
        aeidon.paths.CONFIG_HOME_DIR = mkdir()
        rmdir(aeidon.paths.CONFIG_HOME_DIR)
        aeidon.paths.DATA_HOME_DIR = mkdir()
        rmdir(aeidon.paths.DATA_HOME_DIR)
        aeidon.paths.xdg_copy_if_applicable()
        rmdir(aeidon.paths.OBSOLETE_PROFILE_DIR)



# Copyright (C) 2005-2008 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Enumerations for toolbar style types."""

import aeidon
import gtk

__all__ = ("toolbar_styles",)


class Both(aeidon.EnumerationItem):

    value = gtk.TOOLBAR_BOTH


class BothHoriz(aeidon.EnumerationItem):

    value = gtk.TOOLBAR_BOTH_HORIZ


class Default(aeidon.EnumerationItem):

    value = None


class Icons(aeidon.EnumerationItem):

    value = gtk.TOOLBAR_ICONS


class Text(aeidon.EnumerationItem):

    value = gtk.TOOLBAR_TEXT


toolbar_styles = aeidon.Enumeration()
toolbar_styles.DEFAULT = Default()
toolbar_styles.ICONS = Icons()
toolbar_styles.TEXT = Text()
toolbar_styles.BOTH = Both()
toolbar_styles.BOTH_HORIZ = BothHoriz()



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestSubRip(aeidon.TestCase):

    text = ("All things weird are normal\n"
            "in this whore of cities.")

    def setup_method(self, method):
        self.markup = aeidon.tags.new(aeidon.formats.SUBRIP)

    def test_bolden(self):
        assert self.markup.bolden(self.text) == (
            "<b>All things weird are normal\n"
            "in this whore of cities.</b>")

    def test_clean__closing(self):
        text = ("All<i> things</i> weird are normal\n"
                "in <i> this</i> whore of cities.")
        assert self.markup.clean(text) == (
            "All <i>things</i> weird are normal\n"
            "in <i>this</i> whore of cities.")

    def test_clean__opening(self):
        text = ("All <i>things </i>weird are normal\n"
                "in <i>this </i> whore of cities.")
        assert self.markup.clean(text) == (
            "All <i>things</i> weird are normal\n"
            "in <i>this</i> whore of cities.")

    def test_colorize(self):
        assert self.markup.colorize(self.text, "ccffcc") == (
            '<font color="#ccffcc">All things weird are normal\n'
            'in this whore of cities.</font>')

    def test_decode__bold(self):
        text = ("<B>All things weird are normal\n"
                "in this whore of cities.</B>")
        assert self.markup.decode(text) == (
            "<b>All things weird are normal\n"
            "in this whore of cities.</b>")

    def test_decode__color(self):
        text = ('All things weird are normal\n'
                'in <font color="#ccccff">this whore of cities</font>.')
        assert self.markup.decode(text) == (
            "All things weird are normal\n"
            "in <color=#ccccff>this whore of cities</color>.")

    def test_decode__italic(self):
        text = ("<I>All</I> things weird are normal\n"
                "in this whore of cities.")
        assert self.markup.decode(text) == (
            "<i>All</i> things weird are normal\n"
            "in this whore of cities.")

    def test_decode__underline(self):
        text = ("All things weird are normal\n"
                "in this <U>whore</U> of cities.")
        assert self.markup.decode(text) == (
            "All things weird are normal\n"
            "in this <u>whore</u> of cities.")

    def test_encode__bold(self):
        text = ("<b>All things weird are normal\n"
                "in this whore of cities.</b>")
        assert self.markup.encode(text) == text

    def test_encode__color(self):
        text = ("<color=#ccccff>All things weird are normal\n"
                "in this whore of cities.</color>")
        assert self.markup.encode(text) == (
            '<font color="#ccccff">All things weird are normal\n'
            'in this whore of cities.</font>')

    def test_encode__font(self):
        text = ("<font=Sans>All things weird are normal\n"
                "in this whore of cities.</font>")
        assert self.markup.encode(text) == self.text

    def test_encode__italic(self):
        text = ("All <i>things</i> weird are normal\n"
                "in <i>this</i> whore of cities.")
        assert self.markup.encode(text) == text

    def test_encode__size(self):
        text = ("All things weird are normal\n"
                "in this whore of <size=12>cities</size>.")
        assert self.markup.encode(text) == self.text

    def test_encode__underline(self):
        text = ("All things weird are normal\n"
                "<u>in this whore of cities.</u>")
        assert self.markup.encode(text) == text

    def test_italic_tag(self):
        assert self.markup.italic_tag.match("<i>")
        assert self.markup.italic_tag.match("</i>")

    def test_italicize(self):
        assert self.markup.italicize(self.text, (0, 3)) == (
            "<i>All</i> things weird are normal\n"
            "in this whore of cities.")

    def test_tag(self):
        assert self.markup.tag.match('<font color="#ffccff">')
        assert self.markup.tag.match('</font>')

    def test_underline(self):
        assert self.markup.underline(self.text, (45, 51)) == (
            "All things weird are normal\n"
            "in this whore of <u>cities</u>.")



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk


class TestPositionAgent(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_on_adjust_durations_activate(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_CANCEL
        self.application.get_action("adjust_durations").activate()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_on_convert_framerate_activate(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_CANCEL
        self.application.get_action("convert_framerate").activate()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_on_shift_positions_activate__frame(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_CANCEL
        page = self.application.get_current_page()
        page.edit_mode = aeidon.modes.FRAME
        self.application.get_action("shift_positions").activate()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_on_shift_positions_activate__time(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_CANCEL
        page = self.application.get_current_page()
        page.edit_mode = aeidon.modes.TIME
        self.application.get_action("shift_positions").activate()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_on_speech_recognition_activate(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_CLOSE
        self.application.get_action("speech_recognition").activate()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_on_transform_positions_activate__frame(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_CANCEL
        page = self.application.get_current_page()
        page.edit_mode = aeidon.modes.FRAME
        self.application.get_action("transform_positions").activate()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test_on_transform_positions_activate__time(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_CANCEL
        page = self.application.get_current_page()
        page.edit_mode = aeidon.modes.TIME
        self.application.get_action("transform_positions").activate()



#!/usr/bin/env python

"""
There are four relevant customizations to the standard distutils installation
process: (1) allowing separate installations of aeidon and gaupol, (2) writing
the aeidon.paths module, (3) handling translating of various files and
(4) installing extensions.

(1) Allowing separate installations of aeidon and gaupol are handled through
    global options --with-aeidon, --without-aeidon, --with-gaupol and
    --without-gaupol. See './setup.py --help' and the file 'README.aeidon' for
    documentation and the Distribution class defined in this file for the
    implementation and logic of how that is handled.

(2) Gaupol finds non-Python files based on the paths written in module
    aeidon.paths. In aeidon/paths.py the paths default to the ones in the
    source directory. During the 'install_lib' command the file gets rewritten
    to build/aeidon/paths.py with the installation paths and that file will be
    installed. The paths are based on variable 'install_data' with the 'root'
    variable stripped if it is given. If doing distro-packaging, make sure this
    file gets correctly written.

(3) During installation, the .po files are compiled into .mo files, the desktop
    file, pattern files and extension metadata files are translated. This
    requires gettext and intltool, more specifically, executables 'msgfmt' and
    'intltool-merge' in $PATH.

(4) Extensions are installed under the data directory. All python code included
    in the extensions are compiled during the 'install_data' command, using the
    same arguments for 'byte_compile' as used by the 'install_lib' command. If
    the 'install_lib' command was given '--no-compile' option, then the
    extensions are not compiled either.
"""

import distutils.command.clean
import distutils.command.install
import distutils.command.install_data
import distutils.command.install_lib
import distutils.command.sdist
import glob
import os
import re
import shutil
import sys
import tarfile
import tempfile

os.chdir(os.path.dirname(__file__) or ".")
running_py2exe = "py2exe" in sys.argv

clean = distutils.command.clean.clean
distribution = distutils.dist.Distribution
install = distutils.command.install.install
install_data = distutils.command.install_data.install_data
install_lib = distutils.command.install_lib.install_lib
log = distutils.log
sdist = distutils.command.sdist.sdist

if running_py2exe:
    # py2exe patches the Distribution class, so we need to import py2exe here
    # and subclass the patched Distribution class ourselves.
    # pylint: disable=F0401
    import py2exe
    distribution = py2exe.Distribution

##
## Patch distribution class with new global options.
##

global_options = (
    ("mandir=", None, ("relative installation directory for man pages "
                       "(defaults to 'share/man')")),

    ("with-aeidon", None, "install the aeidon package"),
    ("without-aeidon", None, "don't install the aeidon package"),
    ("with-gaupol", None, "install the gaupol package"),
    ("without-gaupol", None, "don't install the gaupol package"),
    ("with-iso-codes", None, "install iso-codes XML files"),
    ("without-iso-codes", None, "don't install iso-codes XML files"))

negative_opt = {"without-aeidon": "with-aeidon",
                "without-gaupol": "with-gaupol",
                "without-iso-codes": "with-iso-codes"}

distribution.global_options.extend(global_options)
distribution.negative_opt.update(negative_opt)


def get_aeidon_version():
    """Return version number from aeidon/__init__.py."""
    text = open(os.path.join("aeidon", "__init__.py"), "r").read()
    return re.search(r"__version__ *= *['\"](.*?)['\"]", text).group(1)

def get_gaupol_version():
    """Return version number from gaupol/__init__.py."""
    text = open(os.path.join("gaupol", "__init__.py"), "r").read()
    return re.search(r"__version__ *= *['\"](.*?)['\"]", text).group(1)

def run_command_or_exit(command):
    """Run command in shell and raise SystemExit if it fails."""
    if os.system(command) != 0:
        log.error("command %s failed" % repr(command))
        raise SystemExit(1)

def run_command_or_warn(command):
    """Run command in shell and raise SystemExit if it fails."""
    if os.system(command) != 0:
        log.warn("command %s failed" % repr(command))


class Clean(clean):

    """Command to remove files and directories created."""

    def run(self):
        """Remove files and directories listed in setup-files/clean."""
        clean.run(self)
        fobj = open(os.path.join("setup-files", "clean"), "r")
        for targets in (glob.glob(x.strip()) for x in fobj):
            for target in filter(os.path.isdir, targets):
                log.info("removing '%s'" % target)
                if not self.dry_run:
                    shutil.rmtree(target)
            for target in filter(os.path.isfile, targets):
                log.info("removing '%s'" % target)
                if not self.dry_run:
                    os.remove(target)
        fobj.close()


class Distribution(distribution):

    """The core controller of distutils commands."""

    def __find_data_files(self, name):
        """Find data files to install for name."""
        fok = lambda x: not x.endswith((".in", ".pyc"))
        basename = "data-files.%s" % name
        fobj = open(os.path.join("setup-files", basename), "r")
        for line in (x.strip() for x in fobj):
            if not line: continue
            if line.startswith("["):
                dest = line[1:-1]
                continue
            files = filter(fok, glob.glob(line))
            files = filter(os.path.isfile, files)
            assert files
            self.data_files.append((dest, files))
        fobj.close()

    def __find_man_pages(self):
        """Find man pages to install."""
        mandir = self.mandir
        while mandir.endswith("/"):
            mandir = mandir[:-1]
        dest = "/".join((mandir, "man1"))
        self.data_files.append((dest, ("doc/gaupol.1",)))

    def __find_packages(self, name):
        """Find Python packages to install for name."""
        for (root, dirs, files) in os.walk(name):
            init_path = os.path.join(root, "__init__.py")
            if not os.path.isfile(init_path): continue
            path = root.replace(os.sep, ".")
            if path.endswith(".test"): continue
            self.packages.append(path[path.find(name):])

    def __find_scripts(self, name):
        """Find scripts to install for name."""
        if name == "gaupol":
            self.scripts.append("bin/gaupol")

    def __init__(self, attrs=None):
        """Initialize a Distribution object."""
        self.mandir = "share/man"
        self.with_aeidon = True
        self.with_gaupol = True
        self.with_iso_codes = True
        value = distribution.__init__(self, attrs)
        self.data_files = []
        self.packages = []
        self.scripts = []
        return value

    def parse_command_line(self):
        """Parse commands and options given as arguments."""
        value = distribution.parse_command_line(self)
        # Configuration files are parsed before the command line,
        # so once the command line is parsed, all options have
        # their final values and thus files corresponding to
        # the packages to install can finally be set.
        if self.with_aeidon:
            self.__find_data_files("aeidon")
            self.__find_packages("aeidon")
            if self.with_iso_codes:
                self.__find_data_files("iso-codes")
        if self.with_gaupol:
            self.__find_data_files("gaupol")
            self.__find_man_pages()
            self.__find_packages("gaupol")
            self.__find_scripts("gaupol")
        # Redefine name, version and requires metadata attributes. These are
        # used in the egg-info file written by distutils. While egg-info files
        # appear completely useless, it needs to be ensured that if aeidon and
        # gaupol are installed separately, they install differently named
        # egg-info files in order to avoid overwriting files.
        if self.with_gaupol:
            self.metadata.name = "gaupol"
            self.metadata.version = get_gaupol_version()
            self.metadata.requires = ("gtk",)
        else: # Without gaupol.
            self.metadata.name = "aeidon"
            self.metadata.version = get_aeidon_version()
            self.metadata.requires = ()
        return value

    def parse_config_files(self, filenames=None):
        """Parse options from configuration files."""
        value = distribution.parse_config_files(self, filenames)
        strtobool = distutils.util.strtobool
        for option in ("with_aeidon", "with_gaupol", "with_iso_codes"):
            value = getattr(self, option)
            if isinstance(value, basestring):
                setattr(self, option, strtobool(value))
        return value


class Documentation(distutils.cmd.Command):

    """Command to build documentation from source code."""

    description = "build documentation from source code"
    user_options = [("format=", "f",
                     "type of documentation to create (try 'html')")]

    def initialize_options(self):
        """Initialize default values for options."""
        self.format = None

    def finalize_options(self):
        """Ensure that options have valid values."""
        if self.format is None:
            log.warn("format not specified, using 'html'")
            self.format = "html"

    def run(self):
        """Build documentation from source code."""
        os.chdir(os.path.join("doc", "sphinx"))
        if not self.dry_run:
            run_command_or_exit("make clean")
            run_command_or_exit("python%d.%d autogen.py aeidon gaupol"
                                % sys.version_info[:2])

            run_command_or_exit("make %s" % self.format)


class Install(install):

    """Command to install everything."""

    def run(self):
        """Install everything and update the desktop file database."""
        install.run(self)
        if self.distribution.with_gaupol:
            get_command_obj = self.distribution.get_command_obj
            root = get_command_obj("install").root
            data_dir = get_command_obj("install_data").install_dir
            # Assume we're actually installing if --root was not given.
            if (root is not None) or (data_dir is None): return
            directory = os.path.join(data_dir, "share", "applications")
            log.info("updating desktop database in '%s'" % directory)
            run_command_or_warn('update-desktop-database "%s"' % directory)


class InstallData(install_data):

    """Command to install data files."""

    def __build_extensions(self):
        """Build all Python code files in extensions."""
        get_command_obj = self.distribution.get_command_obj
        if not get_command_obj("install_lib").compile: return
        optimize = get_command_obj("install_lib").optimize
        data_dir = get_command_obj("install_data").install_dir
        data_dir = os.path.join(data_dir, "share", "gaupol")
        files = glob.glob("%s/extensions/*/*.py" % data_dir)
        distutils.util.byte_compile(files, optimize, self.force, self.dry_run)

    def __get_desktop_file(self):
        """Return a tuple for translated desktop file."""
        path = os.path.join("data", "gaupol.desktop")
        if not running_py2exe:
            command = "intltool-merge -d po %s.in %s" % (path, path)
            run_command_or_exit(command)
        return ("share/applications", (path,))

    def __get_extension_file(self, extension, extension_file):
        """Return a tuple for translated extension metadata file."""
        assert extension_file.endswith(".in")
        path = extension_file[:-3]
        if not running_py2exe:
            command = "intltool-merge -d po %s.in %s" % (path, path)
            run_command_or_exit(command)
        return ("share/gaupol/extensions/%s" % extension, (path,))

    def __get_mo_file(self, po_file):
        """Return a tuple for compiled .mo file."""
        locale = os.path.basename(po_file[:-3])
        mo_dir = os.path.join("locale", locale, "LC_MESSAGES")
        if not os.path.isdir(mo_dir):
            log.info("creating %s" % mo_dir)
            os.makedirs(mo_dir)
        mo_file = os.path.join(mo_dir, "gaupol.mo")
        dest_dir = os.path.join("share", mo_dir)
        if not running_py2exe:
            log.info("compiling '%s'" % mo_file)
            command = "msgfmt %s -o %s" % (po_file, mo_file)
            run_command_or_exit(command)
        return (dest_dir, (mo_file,))

    def __get_pattern_file(self, pattern_file):
        """Return a tuple for the translated pattern file."""
        assert pattern_file.endswith(".in")
        path = pattern_file[:-3]
        if not running_py2exe:
            command = "intltool-merge -d po %s.in %s" % (path, path)
            run_command_or_exit(command)
        return ("share/gaupol/patterns", (path,))

    def run(self):
        """Install data files after translating them."""
        if self.distribution.with_aeidon:
            for po_file in glob.glob("po/*.po"):
                self.data_files.append(self.__get_mo_file(po_file))
            for pattern_file in glob.glob("data/patterns/*.in"):
                self.data_files.append(self.__get_pattern_file(pattern_file))
        if self.distribution.with_gaupol:
            for extension in os.listdir("data/extensions"):
                pattern = "data/extensions/%s/*.gaupol-extension.in"
                pattern = pattern % extension
                for extension_file in glob.glob(pattern):
                    t = self.__get_extension_file(extension, extension_file)
                    self.data_files.append(t)
            self.data_files.append(self.__get_desktop_file())
        install_data.run(self)
        if self.distribution.with_gaupol:
            self.__build_extensions()


class InstallLib(install_lib):

    """Command to install library files."""

    def __write_paths_module(self):
        """Write installation paths to build/aeidon/paths.py."""
        get_command_obj = self.distribution.get_command_obj
        root = get_command_obj("install").root
        prefix = get_command_obj("install").install_data
        # Allow --root to be used like $DESTDIR.
        if root is not None:
            prefix = os.path.abspath(prefix)
            prefix = prefix.replace(os.path.abspath(root), "")
        data_dir = os.path.join(prefix, "share", "gaupol")
        locale_dir = os.path.join(prefix, "share", "locale")
        # Write changes to the aeidon.paths module.
        path = os.path.join(self.build_dir, "aeidon", "paths.py")
        text = open(path, "r").read()
        patt = "DATA_DIR = get_data_directory()"
        repl = "DATA_DIR = %s" % repr(data_dir)
        text = text.replace(patt, repl)
        assert text.count(repr(data_dir)) > 0
        patt = "LOCALE_DIR = get_locale_directory()"
        repl = "LOCALE_DIR = %s" % repr(locale_dir)
        text = text.replace(patt, repl)
        assert text.count(repr(locale_dir)) > 0
        open(path, "w").write(text)

    def install(self):
        """Install library files after writing changes."""
        if self.distribution.with_aeidon:
            self.__write_paths_module()
        return install_lib.install(self)


class SDistGna(sdist):

    description = "create source distribution for gna.org"

    def finalize_options(self):
        """Set the distribution directory to 'dist/X.Y'."""
        version = get_gaupol_version()
        sdist.finalize_options(self)
        branch = ".".join(version.split(".")[:2])
        self.dist_dir = os.path.join(self.dist_dir, branch)

    def run(self):
        version = get_gaupol_version()
        if os.path.isfile("ChangeLog"):
            os.remove("ChangeLog")
        run_command_or_exit("tools/generate-change-log > ChangeLog")
        assert os.path.isfile("ChangeLog")
        assert open("ChangeLog", "r").read().strip()
        sdist.run(self)
        basename = "gaupol-%s" % version
        tarballs = os.listdir(self.dist_dir)
        os.chdir(self.dist_dir)
        # Compare tarball contents with working copy.
        temp_dir = tempfile.gettempdir()
        test_dir = os.path.join(temp_dir, basename)
        tobj = tarfile.open(tarballs[-1], "r")
        for member in tobj.getmembers():
            tobj.extract(member, temp_dir)
        log.info("comparing tarball (tmp) with working copy (../..)")
        os.system('diff -qr -x ".*" -x "*.pyc" ../.. %s' % test_dir)
        response = raw_input("Are all files in the tarball [Y/n]? ")
        if response.lower() == "n":
            raise SystemExit("Must edit MANIFEST.in")
        shutil.rmtree(test_dir)
        # Create extra distribution files.
        log.info("calculating md5sums")
        run_command_or_exit("md5sum * > %s.md5sum" % basename)
        log.info("creating '%s.changes'" % basename)
        source = os.path.join("..", "..", "ChangeLog")
        shutil.copyfile(source, "%s.changes" % basename)
        log.info("creating '%s.news'" % basename)
        source = os.path.join("..", "..", "NEWS")
        shutil.copyfile(source, "%s.news" % basename)
        for tarball in tarballs:
            log.info("signing '%s'" % tarball)
            run_command_or_exit("gpg --detach %s" % tarball)
        os.chdir("..")
        log.info("creating 'latest.txt'")
        with open("latest.txt", "w") as fobj:
            fobj.write("%s\n" % version)


setup_kwargs = dict(
    name="gaupol",
    version=get_gaupol_version(),
    requires=("gtk",),
    platforms=("Platform Independent",),
    author="Osmo Salomaa",
    author_email="otsaloma@iki.fi",
    url="http://home.gna.org/gaupol/",
    description="Editor for text-based subtitle files",
    license="GPL",
    distclass=Distribution,
    cmdclass={
        "clean": Clean,
        "doc": Documentation,
        "install": Install,
        "install_data": InstallData,
        "install_lib": InstallLib,
        "sdist_gna": SDistGna,
        })

if __name__ == "__main__":
    distutils.core.setup(**setup_kwargs)



# Copyright (C) 2005-2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Widget to display subtitle data in the form of a list."""

import aeidon
import gaupol
import gtk
import pango
import re
_ = aeidon.i18n._

__all__ = ("View",)


class View(gtk.TreeView):

    """Widget to display subtitle data in the form of a list.

    :ivar columns: Enumeration for columns currently shown

       The values of the enumeration items correspond to the column indices and
       are updated when columns are added, removed or reordered. Note that
       these indices are not necessarily the same as the column indices in the
       underlying :class:`gtk.ListStore` data model.
    """

    __metaclass__ = gaupol.ContractualGObject

    def __init__(self, edit_mode):
        """Initialize a :class:`View` object."""
        gtk.TreeView.__init__(self)
        self._active_attr = None
        self._active_col_name = ""
        self._calc = aeidon.Calculator()
        self._normal_attr = None
        self.columns = aeidon.Enumeration()
        self._init_column_attributes()
        self._init_signal_handlers()
        self._init_props(edit_mode)

    def _get_renderer(self, field, edit_mode):
        """Initialize and return a new cell renderer for `field`."""
        font = gaupol.util.get_font()
        if field == gaupol.fields.NUMBER:
            renderer = gtk.CellRendererText()
            renderer.props.xalign = 1
        elif field.is_position:
            if edit_mode == aeidon.modes.TIME:
                if field == gaupol.fields.DURATION:
                    renderer = gaupol.FloatCellRenderer()
                else: renderer = gaupol.TimeCellRenderer()
            elif edit_mode == aeidon.modes.FRAME:
                renderer = gtk.CellRendererText()
            renderer.props.xalign = 1
        elif field.is_text:
            renderer = gaupol.MultilineCellRenderer()
            renderer.props.yalign = 0
        renderer.props.editable = (field != gaupol.fields.NUMBER)
        renderer.props.font = font
        renderer.props.xpad = 4
        return renderer

    def _init_cell_data_functions(self):
        """Initialize functions to automatically update cell data."""
        # Set the data in the number column automatically.
        def set_number(column, renderer, store, itr):
            renderer.props.text = store.get_path(itr)[0] + 1
        column = self.get_column(self.columns.NUMBER)
        renderer = column.get_cell_renderers()[0]
        column.set_cell_data_func(renderer, set_number)

    def _init_column_attributes(self):
        """Initialize the column header :class:`pango.AttrList`."""
        self._active_attr = pango.AttrList()
        attr = pango.AttrWeight(pango.WEIGHT_BOLD, 0, -1)
        self._active_attr.insert(attr)
        self._normal_attr = pango.AttrList()
        attr = pango.AttrWeight(pango.WEIGHT_NORMAL, 0, -1)
        self._normal_attr.insert(attr)

    def _init_columns(self, edit_mode):
        """Initialize the tree view columns."""
        visible_fields = gaupol.conf.editor.visible_fields
        for field in gaupol.conf.editor.field_order:
            renderer = self._get_renderer(field, edit_mode)
            column = gtk.TreeViewColumn(field.label, renderer, text=field)
            column.set_data("identifier", field.name.lower())
            self.append_column(column)
            column.set_clickable(True)
            column.set_resizable(True)
            column.set_reorderable(True)
            column.set_expand(field.is_text)
            column.set_visible(field in visible_fields)
            label = self.get_header_label(field.label)
            label.set_tooltip_text(field.tooltip)
            column.set_widget(label)

    def _init_props(self, edit_mode):
        """Initialize properties."""
        if edit_mode == aeidon.modes.TIME:
            columns = (int, str, str, float, str, str)
        if edit_mode == aeidon.modes.FRAME:
            columns = (int, int, int, int, str, str)
        store = gtk.ListStore(*columns)
        self.set_model(store)
        self._init_columns(edit_mode)
        self._init_cell_data_functions()
        self.set_headers_visible(True)
        self.set_rules_hint(True)
        self.set_rubber_banding(True)
        selection = self.get_selection()
        selection.set_mode(gtk.SELECTION_MULTIPLE)
        self._init_search()

    def _init_search(self):
        """Initialize the interactive search properties."""
        self.set_enable_search(True)
        self.set_search_column(self.columns.NUMBER)
        self.set_search_equal_func(self._search_equals)

    def _init_signal_handlers(self):
        """Initialize signal handlers."""
        aeidon.util.connect(self, self, "columns-changed")
        aeidon.util.connect(self, self, "cursor-changed")
        aeidon.util.connect(self, self, "key-press-event")
        gaupol.conf.connect_notify("editor", "custom_font", self)
        gaupol.conf.connect_notify("editor", "length_unit", self)
        gaupol.conf.connect_notify("editor", "show_lengths_cell", self)
        gaupol.conf.connect_notify("editor", "use_custom_font", self)

    def _on_conf_editor_notify_custom_font(self, *args):
        """Apply the new font to all columns."""
        font = gaupol.util.get_font()
        for column in self.get_columns():
            renderer = column.get_cell_renderers()[0]
            if hasattr(renderer.props, "font"):
                renderer.props.font = font
        self.columns_autosize()

    def _on_conf_editor_notify_length_unit(self, *args):
        """Repaint the cells to update line length display."""
        if gaupol.conf.editor.show_lengths_cell:
            self.columns_autosize()

    def _on_conf_editor_notify_show_lengths_cell(self, *args):
        """Repaint the cells to update line length display."""
        self.columns_autosize()

    def _on_conf_editor_notify_use_custom_font(self, *args):
        """Apply the new font to all columns."""
        font = gaupol.util.get_font()
        for column in self.get_columns():
            renderer = column.get_cell_renderers()[0]
            if hasattr(renderer.props, "font"):
                renderer.props.font = font
        self.columns_autosize()

    def _on_columns_changed(self, *args):
        """Reset the columns enumeration to match new column layout."""
        count_changed = (len(self.columns) != len(self.get_columns()))
        self._reset_columns()
        field_order = []
        for column in self.get_columns():
            identifier = column.get_data("identifier")
            attribute = identifier.upper()
            item = aeidon.EnumerationItem()
            setattr(self.columns, attribute, item)
            if hasattr(gaupol.fields, attribute):
                field = getattr(gaupol.fields, attribute)
                field_order.append(field)
        if not count_changed:
            gaupol.conf.editor.field_order = field_order

    def _on_cursor_changed(self, *args):
        """Update the column header labels to refelect changed focus."""
        self.update_headers()

    def _on_key_press_event(self, widget, event):
        """Handle various special-case key combinations."""
        # Disable Ctrl+PageUp/PageDown to allow them to be
        # used solely for navigation between notebook tabs.
        if event.state & gtk.gdk.CONTROL_MASK:
            if event.keyval in (gtk.keysyms.Page_Up, gtk.keysyms.Page_Down):
                return widget.stop_emission("key-press-event")
        # Use interactive search for a subtitle number or time
        # only if valid string keys have been pressed.
        self.set_enable_search(event.string.isdigit() or
                               event.string == ":" or
                               event.string == ".")

    def _reset_columns(self):
        """Recreate the columns enumeration and set all items to ``None``."""
        # Set default cell enumeration items to None without magic to silence
        # false pylint positives about referencing a missing attribute.
        self.columns = aeidon.Enumeration()
        self.columns.NUMBER = None
        self.columns.START = None
        self.columns.END = None
        self.columns.DURATION = None
        self.columns.MAIN_TEXT = None
        self.columns.TRAN_TEXT = None

    def _search_equals(self, store, column, key, itr):
        """Return ``False`` if `key` matches either subtitle number or time."""
        row = int(store.get_path(itr)[0])
        if key.count(":") == 0:
            # Search for subtitle number.
            try: return row != (int(key) - 1)
            except ValueError: return False
        col = gaupol.conf.editor.field_order.index(gaupol.fields.START)
        hours = minutes = seconds = 0
        if key.count(":") == 1 and key.startswith(":"):
            # Search for time of form ':MM'
            match = re.search(r"^:(\d{1,2})$", key)
            if match is None: return False
            minutes = int(match.group(1))
        if key.count(":") == 1 and not key.startswith(":"):
            # Search for time of form 'MM:[SS.SSS]'
            match = re.search(r"^(\d{1,2}):([\d.]+)?$", key)
            if match is None: return False
            minutes = int(match.group(1))
            try: seconds = float(match.group(2) or "00.000")
            except ValueError: return False
        if key.count(":") == 2:
            # Search for time of form '[HH]:MM:[SS.SSS]'
            match = re.search(r"^(\d{,2}):(\d{1,2}):([\d.]+)?$", key)
            if match is None: return False
            hours = int(match.group(1) or "00")
            minutes = int(match.group(2))
            try: seconds = float(match.group(3) or "00.000")
            except ValueError: return False
        time_key = "%02d:%02d:%06.3f" % (hours, minutes, seconds)
        if not self._calc.is_valid_time(time_key): return False
        time_iter = store[row][col]
        if not ":" in time_iter: return False
        try: time_next = store[row + 1][col]
        except IndexError: time_next = "99:59:59.999"
        return not (self._calc.time_to_seconds(time_iter)
                    <= self._calc.time_to_seconds(time_key)
                    < self._calc.time_to_seconds(time_next))

    def get_focus_ensure(self, value):
        store = self.get_model()
        if value[0] is not None:
            assert 0 <= value[0] < len(store)
        if value[1] is not None:
            assert value[1] in self.columns

    def get_focus(self):
        """Return the row and column of the current focus."""
        row, col = self.get_cursor()
        if row is not None:
            row = row[0]
        if col is not None:
            col = self.get_columns().index(col)
        return row, col

    def get_header_label(self, text):
        """Return a column header label that's wide enough."""
        label = gtk.Label(text)
        label.props.xalign = 0
        label.show()
        label.set_attributes(self._active_attr)
        width = label.size_request()[0]
        label.set_size_request(width, -1)
        label.set_attributes(self._normal_attr)
        return label

    def get_selected_rows_ensure(self, value):
        store = self.get_model()
        for row in value:
            assert 0 <= row < len(store)

    def get_selected_rows(self):
        """Return a sequence of the selected rows."""
        rows = self.get_selection().get_selected_rows()[1]
        return tuple(x[0] for x in rows)

    def is_position_column(self, col):
        """Return ``True`` if `col` is a position column."""
        return col in (self.columns.START,
                       self.columns.END,
                       self.columns.DURATION)

    def is_text_column(self, col):
        """Return True if `col` is a text column."""
        return col in (self.columns.MAIN_TEXT,
                       self.columns.TRAN_TEXT)

    def scroll_to_row_require(self, row):
        store = self.get_model()
        assert 0 <= row < len(store)

    def scroll_to_row(self, row):
        """Scroll view until `row` is visible."""
        self.scroll_to_cell(row, None, True, 0.5, 0)

    def select_rows_require(self, rows):
        store = self.get_model()
        for row in rows:
            assert 0 <= row < len(store)

    def select_rows(self, rows):
        """Select `rows`, clearing previous selection."""
        # Select by ranges to avoid sending too many 'changed' signals.
        selection = self.get_selection()
        selection.unselect_all()
        for lst in aeidon.util.get_ranges(rows):
            selection.select_range(lst[0], lst[-1])

    def set_focus_require(self, row, col=None):
        store = self.get_model()
        assert -1 <= row < len(store)
        if col is not None:
            assert col in self.columns

    def set_focus(self, row, col=None):
        """Set the focus to `row` (-1 for last), `col`."""
        if row == -1:
            row = len(self.get_model()) - 1
        if col is not None:
            col = self.get_column(col)
        self.set_cursor(row, col)

    def update_headers(self):
        """Update the attributes of the column header labels."""
        fcol = self.get_focus()[1]
        acol = getattr(self.columns, self._active_col_name, None)
        if fcol == acol: return
        self._active_col_name = ""
        if acol is not None:
            label = self.get_column(acol).get_widget()
            label.set_attributes(self._normal_attr)
        if fcol is not None:
            label = self.get_column(fcol).get_widget()
            label.set_attributes(self._active_attr)
            self._active_col_name = self.columns[fcol].name



# Copyright (C) 2005-2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Opening subtitle files and creating new projects."""

from __future__ import division

import aeidon
import gaupol
import gtk
import os
_ = aeidon.i18n._


class OpenAgent(aeidon.Delegate):

    """Opening subtitle files and creating new projects."""

    __metaclass__ = aeidon.Contractual

    def _append_subtitles(self, from_page, to_page):
        """Append subtitles in `from_page` to `to_page`.

        Return sequence of new indices in `to_page`.
        """
        fp = from_page.project
        tp = to_page.project
        tn = len(tp.subtitles)
        indices = range(tn, tn + len(fp.subtitles))
        tp.block("action-done")
        tp.insert_subtitles(indices, fp.subtitles)
        tp.set_action_description(aeidon.registers.DO, _("Appending file"))
        tp.unblock("action-done")
        return tuple(indices)

    def _check_file_exists(self, path):
        """Raise :exc:`gaupol.Default` if no file at `path`."""
        gaupol.util.raise_default(not os.path.isfile(path))

    def _check_file_not_open(self, path):
        """Raise :exc:`gaupol.Default` if file at `path` already open."""
        for page in self.pages:
            files = (page.project.main_file, page.project.tran_file)
            paths = [x.path for x in filter(None, files)]
            if not path in paths: continue
            self.set_current_page(page)
            message = _('File "%s" is already open')
            self.flash_message(message % os.path.basename(path))
            raise gaupol.Default

    def _check_file_size(self, path):
        """Raise :exc:`gaupol.Default` if size of file at `path` too large."""
        size_mb = os.stat(path)[6] / 1048576
        if size_mb <= 1: return
        basename = os.path.basename(path)
        self._show_size_warning_dialog(basename, size_mb)

    def _check_sort_count(self, path, sort_count):
        """Raise :exc:`gaupol.Default` if `sort_count` too large."""
        if sort_count <= 0: return
        basename = os.path.basename(path)
        self._show_sort_warning_dialog(basename, sort_count)

    def _get_encodings_require(self, first=None):
        if not first in (None, "auto"):
            assert aeidon.encodings.is_valid_code(first)

    def _get_encodings_ensure(self, value, first=None):
        assert value
        for encoding in filter(lambda x: x != "auto", value):
            assert aeidon.encodings.is_valid_code(encoding)
        if first is not None:
            assert value[0] == first

    def _get_encodings(self, first=None):
        """Return a sequence of encodings to try when opening files."""
        encodings = [first]
        if gaupol.conf.encoding.try_locale:
            encoding = aeidon.encodings.get_locale_code()
            encodings.append(encoding)
        encodings += gaupol.conf.encoding.fallback
        try_auto = gaupol.conf.encoding.try_auto
        if try_auto and aeidon.util.chardet_available():
            encodings.append("auto")
        while None in encodings:
            encodings.remove(None)
        encodings = encodings or ["utf_8"]
        return tuple(aeidon.util.get_unique(encodings))

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_append_file_activate(self, *args):
        """Append subtitles from file to the current project."""
        gaupol.util.set_cursor_busy(self.window)
        dialog = gaupol.AppendDialog(self.window)
        gaupol.util.set_cursor_normal(self.window)
        response = gaupol.util.run_dialog(dialog)
        paths = dialog.get_filenames()
        encoding = dialog.get_encoding()
        dialog.destroy()
        if response != gtk.RESPONSE_OK: return
        if not paths: return
        gaupol.util.iterate_main()
        self.append_file(paths[0], encoding)

    @aeidon.deco.export
    def _on_new_project_activate(self, *args):
        """Create a new project."""
        page = gaupol.Page(self.counter.next())
        page.project.insert_subtitles((0,), register=None)
        self.add_page(page)

    @aeidon.deco.export
    def _on_notebook_drag_data_received(
        self, notebook, context, x, y, selection_data, info, time):
        """Open main files from dragged URIs."""
        uris = selection_data.get_uris()
        paths = map(aeidon.util.uri_to_path, uris)
        self.open_main(paths)

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_open_main_files_activate(self, *args):
        """Open main files."""
        doc = aeidon.documents.MAIN
        paths, encoding = self._select_files(_("Open"), doc)
        self.open_main(paths, encoding)

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_open_main_files_recent_activate(self, *args):
        """Open main files."""
        doc = aeidon.documents.MAIN
        paths, encoding = self._select_files(_("Open"), doc)
        self.open_main(paths, encoding)

    @aeidon.deco.export
    def _on_open_main_files_recent_item_activated(self, chooser):
        """Open a recent main file."""
        uri = chooser.get_current_uri()
        path = aeidon.util.uri_to_path(uri)
        self.open_main(path)

    @aeidon.deco.export
    def _on_open_recent_main_file_item_activated(self, chooser):
        """Open a recent main file."""
        uri = chooser.get_current_uri()
        path = aeidon.util.uri_to_path(uri)
        self.open_main(path)

    @aeidon.deco.export
    def _on_open_recent_translation_file_item_activated(self, chooser):
        """Open a recent translation file."""
        uri = chooser.get_current_uri()
        path = aeidon.util.uri_to_path(uri)
        align_method = aeidon.align_methods.POSITION
        self.open_translation(path, None, align_method)

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_open_translation_file_activate(self, *args):
        """Open a translation file."""
        page = self.get_current_page()
        if page.project.tran_changed:
            self._show_translation_warning_dialog(page)
        doc = aeidon.documents.TRAN
        paths, encoding = self._select_files(_("Open Translation"), doc)
        self.open_translation(paths[0], encoding)

    @aeidon.deco.export
    def _on_select_video_file_activate(self, *args):
        """Select a video file."""
        gaupol.util.set_cursor_busy(self.window)
        page = self.get_current_page()
        path = page.project.video_path
        dialog = gaupol.VideoDialog(self.window)
        if page.project.video_path is not None:
            dialog.set_filename(page.project.video_path)
        elif page.project.main_file is not None:
            directory = os.path.dirname(page.project.main_file.path)
            dialog.set_current_folder(directory)
        gaupol.util.set_cursor_normal(self.window)
        response = gaupol.util.run_dialog(dialog)
        path = dialog.get_filename()
        dialog.destroy()
        if response != gtk.RESPONSE_OK: return
        page.project.video_path = path
        self.update_gui()

    @aeidon.deco.export
    def _on_split_project_activate(self, *args):
        """Split the current project in two."""
        gaupol.util.flash_dialog(gaupol.SplitDialog(self.window, self))

    @aeidon.deco.export
    def _on_video_button_clicked(self, *args):
        """Select a video file."""
        self.get_action("select_video_file").activate()

    @aeidon.deco.export
    def _on_video_button_drag_data_received(
        self, notebook, context, x, y, selection_data, info, time):
        """Set video file from dragged URI."""
        page = self.get_current_page()
        uri = selection_data.get_uris()[0]
        path = aeidon.util.uri_to_path(uri)
        if not os.path.isfile(path): return
        page.project.video_path = path
        self.update_gui()

    def _open_file(self, path, encodings, doc, check_open=True):
        """Open file at `path` and return corresponding page if successful.

        Raise :exc:`Default` if cancelled or file cannot be opened.
        """
        self._check_file_exists(path)
        if check_open:
            self._check_file_not_open(path)
        self._check_file_size(path)
        basename = os.path.basename(path)
        if doc == aeidon.documents.MAIN:
            page = gaupol.Page()
            kwargs = {}
        if doc == aeidon.documents.TRAN:
            page = self.get_current_page()
            kwargs = {"align_method": gaupol.conf.file.align_method}
        for encoding in encodings:
            args = (path, encoding)
            try: sort_count = self._try_open_file(page, doc, *args, **kwargs)
            except UnicodeError: continue
            sfile = page.project.get_file(doc)
            self._check_sort_count(path, sort_count)
            return page
        # Report if all encodings failed to decode file.
        self._show_encoding_error_dialog(basename)
        raise gaupol.Default

    def _select_files(self, title, doc):
        """Show a :class:`gaupol.OpenDialog` to select files.

        Raise :exc:`gaupol.Default` if cancelled.
        """
        gaupol.util.set_cursor_busy(self.window)
        dialog = gaupol.OpenDialog(self.window, title, doc)
        page = self.get_current_page()
        if (page is not None) and (page.project.main_file is not None):
            directory = os.path.dirname(page.project.main_file.path)
            dialog.set_current_folder(directory)
        gaupol.util.set_cursor_normal(self.window)
        response = gaupol.util.run_dialog(dialog)
        paths = dialog.get_filenames()
        encoding = dialog.get_encoding()
        dialog.destroy()
        gaupol.util.raise_default(response != gtk.RESPONSE_OK)
        gaupol.util.iterate_main()
        return paths, encoding

    def _show_encoding_error_dialog(self, basename):
        """Show an error dialog after failing to decode file."""
        title = _('Failed to decode file "%s" with all '
            'attempted codecs') % basename
        message = _("Please try to open the file with a "
            "different character encoding.")
        dialog = gaupol.ErrorDialog(self.window, title, message)
        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        gaupol.util.flash_dialog(dialog)

    def _show_format_error_dialog(self, basename):
        """Show an error dialog after failing to recognize file format."""
        title = _('Failed to recognize format of file "%s"') % basename
        message = _("Please check that the file you are trying to open is a "
            "subtitle file of a format supported by Gaupol.")
        dialog = gaupol.ErrorDialog(self.window, title, message)
        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        gaupol.util.flash_dialog(dialog)

    def _show_io_error_dialog(self, basename, message):
        """Show an error dialog after failing to read file."""
        title = _('Failed to open file "%s"') % basename
        dialog = gaupol.ErrorDialog(self.window, title, message)
        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        gaupol.util.flash_dialog(dialog)

    def _show_parse_error_dialog(self, basename, format):
        """Show an error dialog after failing to parse file."""
        title = _('Failed to parse file "%s"') % basename
        message = _("Please check, e.g. with a text editor, that the file you "
            "are trying open is a valid %s file. If you think it is, file a "
            "bug report and attach the file.") % format.label
        dialog = gaupol.ErrorDialog(self.window, title, message)
        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        gaupol.util.flash_dialog(dialog)

    def _show_size_warning_dialog(self, basename, size):
        """Show a warning dialog when trying to open a large file.

        Raise :exc:`gaupol.Default` if opening cancelled.
        """
        title = _('Open abnormally large file "%s"?') % basename
        message = _("Size of the file is %.1f MB, which is abnormally large "
            "for a text-based subtitle file. Please, check that you are not "
            "trying to open a binary file.") % size
        dialog = gaupol.WarningDialog(self.window, title, message)
        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_NO)
        dialog.add_button(gtk.STOCK_OPEN, gtk.RESPONSE_YES)
        dialog.set_default_response(gtk.RESPONSE_NO)
        response = gaupol.util.flash_dialog(dialog)
        gaupol.util.raise_default(response != gtk.RESPONSE_YES)

    def _show_sort_warning_dialog(self, basename, count):
        """Show a warning dialog when subtitles have been sorted.

        Raise :exc:`gaupol.Default` if opening cancelled.
        """
        title = _('Open unsorted file "%s"?') % basename
        message = _("The order of %d subtitles needs to be changed. "
            "If %d sounds like a lot, the file may be erroneously composed.")
        message = message % (count, count)
        dialog = gaupol.WarningDialog(self.window, title, message)
        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_NO)
        dialog.add_button(gtk.STOCK_OPEN, gtk.RESPONSE_YES)
        dialog.set_default_response(gtk.RESPONSE_YES)
        response = gaupol.util.flash_dialog(dialog)
        gaupol.util.raise_default(response != gtk.RESPONSE_YES)

    def _show_translation_warning_dialog(self, page):
        """Show a warning dialog if opening a new translation file.

        Raise :exc:`gaupol.Default` if opening cancelled.
        """
        title = _('Save changes to translation document "%s" before '
            'opening a new one?') % page.get_translation_basename()
        message = _("If you don't save, changes will be permanently lost.")
        dialog = gaupol.WarningDialog(self.window, title, message)
        dialog.add_button(_("Open _Without Saving"), gtk.RESPONSE_NO)
        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dialog.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_YES)
        dialog.set_default_response(gtk.RESPONSE_YES)
        response = gaupol.util.flash_dialog(dialog)
        if response == gtk.RESPONSE_YES:
            return self.save_translation(page)
        gaupol.util.raise_default(response != gtk.RESPONSE_NO)

    def _try_open_file(self, page, doc, path, encoding, **kwargs):
        """Try to open file at `path` and return subtitle sort count.

        Raise :exc:`gaupol.Default` if reading or parsing fails.
        Raise :exc:`UnicodeError`if decoding fails.
        """
        gaupol.util.set_cursor_busy(self.window)
        basename = os.path.basename(path)
        if encoding == "auto":
            encoding = aeidon.encodings.detect(path)
            if encoding is None: raise UnicodeError
        try: return page.project.open(doc, path, encoding, **kwargs)
        except aeidon.FormatError:
            gaupol.util.set_cursor_normal(self.window)
            self._show_format_error_dialog(basename)
        except IOError as (no, message):
            gaupol.util.set_cursor_normal(self.window)
            self._show_io_error_dialog(basename, message)
        except aeidon.ParseError:
            gaupol.util.set_cursor_normal(self.window)
            bom_encoding = aeidon.encodings.detect_bom(path)
            encoding = bom_encoding or encoding
            silent = aeidon.deco.silent(Exception)
            format = silent(aeidon.util.detect_format)(path, encoding)
            self._show_parse_error_dialog(basename, format)
        finally:
            gaupol.util.set_cursor_normal(self.window)
        raise gaupol.Default

    @aeidon.deco.export
    def add_page(self, page):
        """Add `page` to the application."""
        self.pages.append(page)
        page.connect("close-request", self._on_page_close_request)
        page.project.connect("action-done", self._on_project_action_done)
        page.project.connect("action-redone", self._on_project_action_redone)
        page.project.connect("action-undone", self._on_project_action_undone)
        page.tab_widget.connect("button-press-event",
                                self._on_tab_widget_button_press_event,
                                page)

        self.connect_view_signals(page.view)
        page.project.clipboard.set_texts(self.clipboard.get_texts())
        scroller = gtk.ScrolledWindow()
        scroller.set_policy(*((gtk.POLICY_AUTOMATIC,) * 2))
        scroller.add(page.view)
        self.notebook.append_page(scroller, page.tab_widget)
        self.notebook.set_tab_reorderable(scroller, True)
        self.notebook.show_all()
        self.set_current_page(page)
        self.emit("page-added", page)

    @aeidon.deco.export
    def add_to_recent_files(self, path, format, doc):
        """Add `path` to recent files managed by the recent manager."""
        uri = aeidon.util.path_to_uri(path)
        if doc == aeidon.documents.MAIN:
            group = "gaupol-main"
        if doc == aeidon.documents.TRAN:
            group = "gaupol-translation"
        self.recent_manager.add_full(uri, {"mime_type": format.mime_type,
                                           "app_name": "gaupol",
                                           "app_exec": "gaupol %F",
                                           "groups": (group,),})

    @aeidon.deco.export
    def append_file(self, path, encoding=None):
        """Append subtitles from file at `path` to the current project.

        Raise :exc:`gaupol.Default` if cancelled or something goes wrong.
        """
        encodings = self._get_encodings(encoding)
        doc = aeidon.documents.MAIN
        temp = self._open_file(path, encodings, doc, False)
        gaupol.util.set_cursor_busy(self.window)
        current = self.get_current_page()
        offset = current.project.subtitles[-1].end
        temp.project.shift_positions(None, offset)
        rows = self._append_subtitles(temp, current)
        amount = len(rows)
        current.view.set_focus(rows[0], None)
        current.view.select_rows(rows)
        current.view.scroll_to_row(rows[0])
        basename = temp.get_main_basename()
        message = _('Appended %(amount)d subtitles from "%(basename)s"')
        self.flash_message(message % locals())
        gaupol.util.set_cursor_normal(self.window)

    @aeidon.deco.export
    def connect_view_signals(self, view):
        """Connect to signals emitted by `view`."""
        selection = view.get_selection()
        selection.connect("changed", self._on_view_selection_changed)
        view.connect_after("move-cursor", self._on_view_move_cursor)
        view.connect("button-press-event", self._on_view_button_press_event)
        for column in view.get_columns():
            renderer = column.get_cell_renderers()[0]
            callback = self._on_view_renderer_edited
            renderer.connect("edited", callback, column)
            callback = self._on_view_renderer_editing_started
            renderer.connect("editing-started", callback, column)
            callback = self._on_view_renderer_editing_canceled
            renderer.connect("editing-canceled", callback, column)
            button = column.get_widget().get_ancestor(gtk.Button)
            callback = self._on_view_header_button_press_event
            button.connect("button-press-event", callback)

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def open_main(self, path, encoding=None):
        """Open file at `path` as a main file.

        `path` can be a sequence of paths to open multiple files.
        """
        if isinstance(path, (list, set, tuple)):
            return map(lambda x: self.open_main(x, encoding), sorted(path))
        encodings = self._get_encodings(encoding)
        page = self._open_file(path, encodings, aeidon.documents.MAIN)
        gaupol.util.set_cursor_busy(self.window)
        self.add_page(page)
        format = page.project.main_file.format
        self.add_to_recent_files(path, format, aeidon.documents.MAIN)
        basename = page.get_main_basename()
        self.flash_message(_('Opened main file "%s"') % basename)
        gaupol.util.iterate_main()
        gaupol.util.set_cursor_normal(self.window)

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def open_translation(self, path, encoding=None, align_method=None):
        """Open file at `path` as a translation file."""
        if align_method is not None:
            gaupol.conf.file.align_method = align_method
        encodings = self._get_encodings(encoding)
        page = self._open_file(path, encodings, aeidon.documents.TRAN)
        gaupol.util.set_cursor_busy(self.window)
        col = page.view.columns.TRAN_TEXT
        if not page.view.get_column(col).get_visible():
            self.get_column_action(gaupol.fields.TRAN_TEXT).activate()
        format = page.project.tran_file.format
        self.add_to_recent_files(path, format, aeidon.documents.TRAN)
        basename = page.get_translation_basename()
        self.flash_message(_('Opened translation file "%s"') % basename)
        gaupol.util.set_cursor_normal(self.window)



# Copyright (C) 2006-2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol


class TestAttrDict(gaupol.TestCase):

    # pylint: disable=E1101

    def setup_method(self, method):
        self.root = {"test": 1, "rest": 0}
        self.attr_dict = gaupol.AttributeDictionary(self.root)

    def test___getattr__(self):
        assert self.attr_dict.test == 1
        assert self.attr_dict.rest == 0

    def test___setattr__(self):
        self.attr_dict.test = 5
        assert self.attr_dict.test == 5
        assert self.root["test"] == 5

    def test_add_attribute(self):
        self.attr_dict.add_attribute("best", 2)
        assert self.attr_dict.best == 2
        assert self.root["best"] == 2

    def test_add_attribute__dict(self):
        root = {"test": 1, "rest": 0}
        self.attr_dict.add_attribute("nest", root)
        assert self.attr_dict.nest.test == 1
        assert self.root["nest"]["test"] == 1

    def test_extend(self):
        root = {"test": 3, "pest": 4}
        self.attr_dict.extend(root)
        assert self.attr_dict.test == 1
        assert self.attr_dict.pest == 4
        assert self.attr_dict.rest == 0
        assert self.root["test"] == 1
        assert self.root["pest"] == 4
        assert self.root["rest"] == 0

    def test_extend__dict(self):
        root = {"test": 1, "rest": 0}
        self.attr_dict.add_attribute("nest", root)
        root = {"nest": {"test": 3, "pest": 4}}
        self.attr_dict.extend(root)
        assert self.attr_dict.nest.test == 1
        assert self.attr_dict.nest.pest == 4
        assert self.attr_dict.nest.rest == 0
        assert self.root["nest"]["test"] == 1
        assert self.root["nest"]["pest"] == 4
        assert self.root["nest"]["rest"] == 0

    def test_remove_attribute(self):
        self.attr_dict.remove_attribute("test")
        assert not hasattr(self.attr_dict, "test")
        assert "test" not in self.root

    def test_update(self):
        root = {"test": 3, "pest": 4}
        self.attr_dict.update(root)
        assert self.attr_dict.test == 3
        assert self.attr_dict.pest == 4
        assert self.attr_dict.rest == 0
        assert self.root["test"] == 3
        assert self.root["pest"] == 4
        assert self.root["rest"] == 0

    def test_update__dict(self):
        root = {"test": 1, "rest": 0}
        self.attr_dict.add_attribute("nest", root)
        root = {"nest": {"test": 3, "pest": 4}}
        self.attr_dict.update(root)
        assert self.attr_dict.nest.test == 3
        assert self.attr_dict.nest.pest == 4
        assert self.attr_dict.nest.rest == 0
        assert self.root["nest"]["test"] == 3
        assert self.root["nest"]["pest"] == 4
        assert self.root["nest"]["rest"] == 0



# Copyright (C) 2005-2008,2010-2011 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for displaying a traceback in case of an unhandled exception."""

# This file has been originally adapted from Gazpacho with copyright notice
# Copyright (C) 2005 by Async Open Source and Sicem S.L.

import aeidon
import gaupol
import glib
import gtk
import linecache
import os
import pango
import platform
import string
import sys
import traceback
_ = aeidon.i18n._

__all__ = ("DebugDialog",)


class DebugDialog(gaupol.BuilderDialog):

    """Dialog for displaying a traceback in case of an unhandled exception."""

    _widgets = ("message_label", "text_view")

    def __init__(self):
        """Initialize a :class:`DebugDialog` object."""
        gaupol.BuilderDialog.__init__(self, "debug-dialog.ui")
        self._init_text_tags()
        self._init_signal_handlers()
        self._dialog.set_default_response(gtk.RESPONSE_CLOSE)

    def _init_signal_handlers(self):
        """Initialize signal handlers."""
        aeidon.util.connect(self, self, "response")
        aeidon.util.connect(self, "_text_view", "motion-notify-event")

    def _init_text_tags(self):
        """Initialize tags for the text buffer."""
        text_buffer = self._text_view.get_buffer()
        text_buffer.create_tag("bold", weight=pango.WEIGHT_BOLD)
        text_buffer.create_tag("large", scale=pango.SCALE_LARGE)
        text_buffer.create_tag("monospace", family="monospace")

    def _insert_environment(self):
        """Insert environment information."""
        map(self._insert_text,
            ("Platform: %s\n" % platform.platform(True),
             "Locale: %s.%s\n" % (aeidon.locales.get_system_code(),
                                  aeidon.encodings.get_locale_code()),

             "\n"))

    def _insert_library_versions(self):
        """Insert version numbers of libraries."""
        map(self._insert_text,
            ("Python: %d.%d.%d\n" % sys.version_info[:3],
             "GTK+: %d.%d.%d\n" % gtk.gtk_version,
             "GStreamer: %s\n" % gaupol.util.get_gst_version(),
             "\n"))

    def _insert_link(self, path, lineno, *tags):
        """Insert `path` as a link into the text view."""
        text_buffer = self._text_view.get_buffer()
        tag = text_buffer.create_tag(None, foreground="blue")
        tag.props.underline = pango.UNDERLINE_SINGLE
        tag.connect("event", self._on_text_view_link_tag_event)
        path = os.path.abspath(path)
        tag.set_data("path", path)
        tag.set_data("lineno", lineno)
        if path.startswith(os.getcwd()):
            path = path.replace(os.getcwd(), "")
        while path.startswith(os.sep):
            path = path.replace(os.sep, "", 1)
        itr = text_buffer.get_end_iter()
        tag_table = text_buffer.get_tag_table()
        tags = map(tag_table.lookup, tags + ("monospace",))
        text_buffer.insert_with_tags(itr, path, tag, *tags)

    def _insert_python_package_versions(self):
        """Insert version numbers of Python packages."""
        map(self._insert_text,
            ("aeidon: %s\n" % aeidon.__version__,
             "gaupol: %s\n" % gaupol.__version__,
             "gtk: %d.%d.%d\n" % gtk.pygtk_version,
             "gst: %s\n" % gaupol.util.get_pygst_version(),
             "enchant: %s\n" % aeidon.util.get_enchant_version(),
             "chardet: %s\n" % aeidon.util.get_chardet_version(),
             ))

    def _insert_text(self, text, *tags):
        """Insert `text` with `tags` to the text view."""
        text_buffer = self._text_view.get_buffer()
        itr = text_buffer.get_end_iter()
        tags = tags + ("monospace",)
        text_buffer.insert_with_tags_by_name(itr, text, *tags)

    def _insert_title(self, text):
        """Insert `text` as a title to the text view."""
        self._insert_text(text, "large", "bold")
        self._insert_text("\n\n")

    def _insert_traceback(self, exctype, value, tb, limit=100):
        """Insert up to `limit` stack trace entries from `tb`."""
        for i in range(limit):
            if tb is None: break
            frame = tb.tb_frame
            code = frame.f_code
            line = linecache.getline(code.co_filename,
                                     tb.tb_lineno).strip()

            self._insert_text("File: ")
            self._insert_link(code.co_filename, tb.tb_lineno)
            self._insert_text("\n")
            self._insert_text("Line: %s\n" % str(tb.tb_lineno))
            self._insert_text("In: %s\n\n" % code.co_name)
            if line.strip():
                indent = "\302\240" * 4
                self._insert_text("%s%s\n\n" % (indent, line))
            tb = tb.tb_next
        exception = traceback.format_exception_only(exctype, value)[0]
        exception, space, message = exception.partition(" ")
        self._insert_text(exception, "bold")
        self._insert_text("%s%s\n" % (space, message))

    def _on_editor_exit(self, pid, return_value, command):
        """Print an error message if editor process failed."""
        if return_value == 0: return
        print ("Command '%s' failed with return value %d"
               % (command, return_value))

    def _on_response(self, dialog, response):
        """Do not send response if reporting bug."""
        if response != gtk.RESPONSE_YES: return
        gaupol.util.show_uri(gaupol.BUG_REPORT_URL)
        self.stop_emission("response")

    def _on_text_view_link_tag_event(self, tag, text_view, event, itr):
        """Open linked file in editor."""
        if event.type != gtk.gdk.BUTTON_RELEASE: return
        if event.button != 1: return
        text_buffer = self._text_view.get_buffer()
        assert not text_buffer.get_selection_bounds()
        self._open_link(tag)

    def _on_text_view_motion_notify_event(self, text_view, event):
        """Change mouse pointer when hovering over a link."""
        x = int(event.x)
        y = int(event.y)
        window = gtk.TEXT_WINDOW_WIDGET
        x, y = text_view.window_to_buffer_coords(window, x, y)
        window = text_view.get_window(gtk.TEXT_WINDOW_TEXT)
        for tag in text_view.get_iter_at_location(x, y).get_tags():
            if tag.get_data("path") is not None:
                window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
                return text_view.window.get_pointer()
        window.set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
        text_view.window.get_pointer()

    def _open_link(self, tag):
        """Open linked file in editor."""
        path = aeidon.util.shell_quote(tag.get_data("path"))
        command = string.Template(gaupol.conf.debug.text_editor)
        command = command.safe_substitute(LINENO=tag.get_data("lineno"),
                                          FILE=path)

        process = aeidon.util.start_process(command)
        glib.child_watch_add(process.pid, self._on_editor_exit, command)
        tag.props.foreground = "purple"

    def set_text(self, exctype, value, tb):
        """Set text from `tb` to the text view."""
        self._insert_title("Traceback")
        self._insert_traceback(exctype, value, tb)
        self._insert_title("Environment")
        self._insert_environment()
        self._insert_title("Libraries")
        self._insert_library_versions()
        self._insert_title("Python Packages")
        self._insert_python_package_versions()
        gaupol.util.scale_to_content(self._text_view,
                                     min_nchar=72,
                                     min_nlines=10,
                                     max_nchar=100,
                                     max_nlines=30,
                                     font="monospace")



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Miscellaneous functions and decorators."""

from __future__ import division

import aeidon
import gaupol
import glib
import gtk
import inspect
import pango
import sys
import traceback
import webbrowser


def char_to_px(nchar, font=None):
    """Convert characters to pixels."""
    if nchar < 0: return nchar
    label = gtk.Label("etaoin shrdlu")
    if font is not None:
        set_label_font(label, font)
    width = label.get_layout().get_pixel_size()[0]
    return int(round(nchar * (width / 13)))

def delay_add(delay, function, *args, **kwargs):
    """Call `function` with `args` and `kwargs` once after `delay` (ms).

    Return integer ID of the event source from :func:`glib.timeout_add`.
    """
    def call_function(*args, **kwargs):
        function(*args, **kwargs)
        return False
    return glib.timeout_add(delay, call_function, *args, **kwargs)

def document_to_text_field(doc):
    """Return :attr:`gaupol.fields` item corresponding to `doc`."""
    if doc == aeidon.documents.MAIN:
        return gaupol.fields.MAIN_TEXT
    if doc == aeidon.documents.TRAN:
        return gaupol.fields.TRAN_TEXT
    raise ValueError("Invalid document: %s" % repr(doc))

def flash_dialog(dialog):
    """Run `dialog`, destroy it and return response.

    This function is to be used always when a :class:`gtk.Dialog` is run so
    that unit tests can monkey-patch this function with one that returns a
    specified response without waiting for user input.
    """
    response = dialog.run()
    dialog.destroy()
    return response

def get_font():
    """Return custom font or blank string."""
    return (gaupol.conf.editor.custom_font if
            (gaupol.conf.editor.use_custom_font and
             gaupol.conf.editor.custom_font) else "")

def get_gst_version():
    """Return :mod:`gst` version number as string or ``None``."""
    try:
        import gst
        return ".".join(map(str, gst.version()))
    except Exception:
        return None

def get_preview_command():
    """Return command to use for lauching video player."""
    if gaupol.conf.preview.use_custom_command:
        return gaupol.conf.preview.custom_command
    if gaupol.conf.preview.force_utf_8:
        return gaupol.conf.preview.player.command_utf_8
    return gaupol.conf.preview.player.command

def get_pygst_version():
    """Return ``pygst`` version number as string or ``None``."""
    try:
        import gst
        return ".".join(map(str, gst.pygst_version))
    except Exception:
        return None

def get_text_view_size(text_view, font=""):
    """Return the width and height desired by `text_view`."""
    text_buffer = text_view.get_buffer()
    bounds = text_buffer.get_bounds()
    text = text_buffer.get_text(*bounds)
    label = gtk.Label(text)
    set_label_font(label, font)
    return label.size_request()

def get_tree_view_size(tree_view):
    """Return the width and height desired by `tree_view`."""
    scroller = tree_view.get_parent()
    policy = scroller.get_policy()
    scroller.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
    width, height = scroller.size_request()
    scroller.set_policy(*policy)
    return width, height

@aeidon.deco.once
def gst_available():
    """Return ``True`` if :mod:`gst` module is available."""
    try:
        import gst
        return True
    except Exception:
        return False

@aeidon.deco.once
def gtkspell_available():
    """Return ``True`` if :mod:`gtkspell` module is available."""
    try:
        import gtkspell
        return True
    except Exception:
        return False

def install_module(name, obj):
    """Install `obj`'s module into the :mod:`gaupol` namespace.

    Typical call is of form::

        gaupol.util.install_module("foo", lambda: None)
    """
    gaupol.__dict__[name] = inspect.getmodule(obj)

def iterate_main():
    """Iterate the GTK+ main loop while events are pending."""
    while gtk.events_pending():
        gtk.main_iteration()

def lines_to_px(nlines, font=None):
    """Convert lines to pixels."""
    if nlines < 0: return nlines
    label = gtk.Label("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz")
    if font is not None:
        set_label_font(label, font)
    height = label.get_layout().get_pixel_size()[1]
    return int(round(nlines * height))

@aeidon.deco.once
def pocketsphinx_available():
    """Return ``True`` if `pocketsphinx` gstreamer plugin is available."""
    try:
        import gst
        return gst.plugin_load_by_name("pocketsphinx") is not None
    except Exception:
        return False

def prepare_text_view(text_view):
    """Set spell-check, line-length margin and font properties."""
    if gaupol.util.gtkspell_available() and gaupol.conf.spell_check.inline:
        import gtkspell
        spell = gtkspell.Spell(text_view)
        try: spell.set_language(gaupol.conf.spell_check.language)
        except Exception: spell.detach()

    connect = gaupol.conf.editor.connect
    def update_margin(section, value, text_view):
        if gaupol.conf.editor.show_lengths_edit:
            return gaupol.ruler.connect_text_view(text_view)
        return gaupol.ruler.disconnect_text_view(text_view)

    connect("notify::show_lengths_edit", update_margin, text_view)
    update_margin(None, None, text_view)
    def update_font(section, value, text_view):
        set_widget_font(text_view, get_font())

    connect("notify::use_custom_font", update_font, text_view)
    connect("notify::custom_font", update_font, text_view)
    update_font(None, None, text_view)
    def update_spacing(section, value, text_view):
        if gaupol.conf.editor.show_lengths_cell:
            return text_view.set_pixels_above_lines(2)
        return text_view.set_pixels_above_lines(0)

    connect("notify::show_lengths_cell", update_spacing, text_view)
    update_spacing(None, None, text_view)

def raise_default(expression):
    """Raise :exc:`gaupol.Default` if expression evaluates to ``True``."""
    if expression:
        raise gaupol.Default

def run_dialog(dialog):
    """Run `dialog` and return response.

    This function is to be used always when a :class:`gtk.Dialog` is run so
    that unit tests can monkey patch this function with one that returns a
    specified response without waiting for user input.
    """
    return dialog.run()

def scale_to_content(container,
                     min_nchar=None,
                     min_nlines=None,
                     max_nchar=None,
                     max_nlines=None,
                     font=None):

    """Set `container`'s size by content, but limited by `min` and `max`."""
    # Vaguely account for possible scrollbars.
    bump = lambda x: x + 36
    if isinstance(container, gtk.TextView):
        width, height = map(bump, get_text_view_size(container))
    elif isinstance(container, gtk.TreeView):
        width, height = map(bump, get_tree_view_size(container))
    else:
        raise ValueError("Don't know what to do with container of type %s"
                         % repr(type(container)))

    if min_nchar is not None:
        min_width = char_to_px(min_nchar, font)
        width = max(width, min_width)
    if max_nchar is not None:
        max_width = char_to_px(max_nchar, font)
        width = min(width, max_width)
    if min_nlines is not None:
        min_height = lines_to_px(min_nlines, font)
        height = max(height, min_height)
    if max_nlines is not None:
        max_height = lines_to_px(max_nlines, font)
        height = min(height, max_height)
    container.set_size_request(width, height)

def scale_to_size(widget, nchar, nlines, font=None):
    """Set `widget`'s size to `nchar` and `nlines`."""
    widget.set_size_request(char_to_px(nchar, font),
                            lines_to_px(nlines, font))

def separate_combo(store, itr):
    """Separator function for combo box models."""
    return store.get_value(itr, 0) == gaupol.COMBO_SEPARATOR

def set_cursor_busy_require(window):
    assert hasattr(window, "window")

@aeidon.deco.contractual
def set_cursor_busy(window):
    """Set cursor busy when above window."""
    window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
    iterate_main()

def set_cursor_normal_require(window):
    assert hasattr(window, "window")

@aeidon.deco.contractual
def set_cursor_normal(window):
    """Set cursor normal when above window."""
    window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))
    iterate_main()

def set_label_font(label, font):
    """Use `font` for `label`."""
    context = label.get_pango_context()
    font_desc = context.get_font_description()
    custom_font_desc = pango.FontDescription(font)
    font_desc.merge(custom_font_desc, True)
    attr = pango.AttrFontDesc(font_desc, 0, -1)
    attr_list = pango.AttrList()
    attr_list.insert(attr)
    label.set_attributes(attr_list)

def set_widget_font(widget, font):
    """Use `font` for `widget`."""
    context = widget.get_pango_context()
    font_desc = context.get_font_description()
    custom_font_desc = pango.FontDescription(font)
    font_desc.merge(custom_font_desc, True)
    widget.modify_font(font_desc)

def show_exception(exctype, value, tb):
    """Show exception traceback in :class:`gaupol.DebugDialog`.

    This function can be set as a :func:`sys.excepthook`.
    """
    traceback.print_exception(exctype, value, tb)
    if not isinstance(value, Exception): return
    try: # Avoid recursion.
        dialog = gaupol.DebugDialog()
        dialog.set_text(exctype, value, tb)
        response = dialog.run()
        dialog.destroy()
        if response == gtk.RESPONSE_NO:
            raise SystemExit(1)
    except Exception:
        traceback.print_exc()

def show_uri(uri):
    """Open `uri` in default application."""
    if sys.platform == "win32":
        if uri.startswith(("http://", "https://")):
            # gtk.show_uri (GTK+ 2.20) fails on Windows.
            # GError: No application is registered as handling this file
            return webbrowser.open(uri)
    return gtk.show_uri(None, uri, gtk.gdk.CURRENT_TIME)

def text_field_to_document(field):
    """Return :attr:`aeidon.documents` item corresponding to `field`."""
    if field == gaupol.fields.MAIN_TEXT:
        return aeidon.documents.MAIN
    if field == gaupol.fields.TRAN_TEXT:
        return aeidon.documents.TRAN
    raise ValueError("Invalid field: %s" % repr(field))



# Copyright (C) 2011 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Using custom, non-standard framerates."""

import aeidon
import gaupol
import gtk
import os
_ = aeidon.i18n._


class AddFramerateDialog(gaupol.BuilderDialog):

    """Dialog for entering a custom framerate value."""

    _widgets = ("spin_button",)

    def __init__(self, parent):
        """Initialize an :class:`AddFramerateDialog` object."""
        directory = os.path.abspath(os.path.dirname(__file__))
        ui_file_path = os.path.join(directory, "add-framerate-dialog.ui")
        gaupol.BuilderDialog.__init__(self, ui_file_path)
        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_CLOSE)

    def _on_response(self, *args):
        """Update spin button before dispatching response."""
        self._spin_button.update()

    def get_framerate(self):
        """Return framerate entered in the spin button."""
        return float(self._spin_button.get_value())


class PreferencesDialog(gaupol.BuilderDialog):

    """Dialog for editing list of custom framerates."""

    _widgets = ("add_button", "remove_button", "tree_view")

    def __init__(self, framerates, parent):
        """Initialize a :class:`PreferencesDialog` object."""
        directory = os.path.abspath(os.path.dirname(__file__))
        ui_file_path = os.path.join(directory, "preferences-dialog.ui")
        gaupol.BuilderDialog.__init__(self, ui_file_path)
        self._init_tree_view(framerates)
        self._remove_button.set_sensitive(False)
        gaupol.util.scale_to_content(self._tree_view,
                                     min_nchar=20,
                                     min_nlines=12,
                                     max_nchar=80,
                                     max_nlines=16)

        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_CLOSE)

    def _get_selected_rows(self):
        """Return a sequence of the selected rows."""
        rows = self._tree_view.get_selection().get_selected_rows()[1]
        return tuple(x[0] for x in rows)

    def _init_tree_view(self, framerates):
        """Initialize the tree view."""
        selection = self._tree_view.get_selection()
        selection.set_mode(gtk.SELECTION_MULTIPLE)
        selection.connect("changed", self._on_tree_view_selection_changed)
        store = gtk.ListStore(float)
        for framerate in framerates:
            store.append((framerate,))
        store.set_sort_column_id(0, gtk.SORT_ASCENDING)
        self._tree_view.set_model(store)
        renderer = gtk.CellRendererText()
        renderer.props.xalign = 1
        column = gtk.TreeViewColumn("", renderer, text=0)
        column.set_sort_column_id(0)
        def format_framerate(column, renderer, store, itr):
            renderer.props.text = "%.6f" % store.get_value(itr, 0)
        column.set_cell_data_func(renderer, format_framerate)
        self._tree_view.append_column(column)

    def _on_add_button_clicked(self, *args):
        """Add a new framerate."""
        dialog = AddFramerateDialog(self._dialog)
        response = gaupol.util.run_dialog(dialog)
        framerate = dialog.get_framerate()
        dialog.destroy()
        if response != gtk.RESPONSE_OK: return
        store = self._tree_view.get_model()
        store.append((framerate,))

    def _on_remove_button_clicked(self, *args):
        """Remove the selected framerate."""
        rows = self._get_selected_rows()
        store = self._tree_view.get_model()
        for row in reversed(sorted(rows)):
            store.remove(store.get_iter(row))
        if len(store) <= 0: return
        self._tree_view.set_cursor(max(row - 1, 0))

    def _on_tree_view_selection_changed(self, *args):
        """Set the remove button sensitivity."""
        selection = self._tree_view.get_selection()
        n = selection.count_selected_rows()
        self._remove_button.set_sensitive(n > 0)

    def get_framerates(self):
        """Return the defined custom framerates."""
        framerates = []
        store = self._tree_view.get_model()
        for i in range(len(store)):
            framerates.append(store[i][0])
        return(tuple(sorted(framerates)))


class CustomFrameratesExtension(gaupol.Extension):

    """Using custom, non-standard framerates."""

    def __init__(self):
        """Initialize a :class:`CustomFrameratesExtension` object."""
        self._action_group = None
        self._conf = None
        self._framerates = []
        self._uim_ids = []
        self.application = None

    def _clear_attributes(self):
        """Clear values of attributes."""
        self._action_group = None
        self._conf = None
        self._framerates = []
        self._uim_ids = []
        self.application = None

    def _add_framerates(self):
        """Add custom framerates and corresponding UI elements."""
        self._action_group = gtk.ActionGroup("custom-framerates")
        self.application.uim.insert_action_group(self._action_group, -1)
        tooltip = _("Calculate nonnative units with a framerate of %.3f fps")
        directory = os.path.abspath(os.path.dirname(__file__))
        ui_file_path = os.path.join(directory, "custom-framerates.ui.xml")
        ui_xml_template = open(ui_file_path, "r").read()
        self._framerates = []
        self._uim_ids = []
        for value in sorted(self._conf.framerates):
            name = "FPS_%s" % (("%.3f" % value).replace(".", "_"))
            if hasattr(aeidon.framerates, name):
                print "Framerate %.3f already exists!" % value
                continue
            setattr(aeidon.framerates, name, aeidon.EnumerationItem())
            framerate = getattr(aeidon.framerates, name)
            framerate.label = "%.3f fps" % value
            framerate.mpsub = "%.2f" % value
            framerate.value = float(value)
            self._framerates.append(framerate)
            action_name = name.replace("FPS", "show_framerate")
            action = gtk.RadioAction(name=action_name,
                                     label=framerate.label,
                                     tooltip=(tooltip % value),
                                     stock_id=None,
                                     value=int(framerate))

            group = "show_framerate_23_976"
            action.set_group(self.application.get_action(group))
            action.framerate = framerate
            self._action_group.add_action(action)
            ui_xml = ui_xml_template % (name.replace("FPS_", ""),
                                        action.get_name())

            uim_id = self.application.uim.add_ui_from_string(ui_xml)
            self._uim_ids.append(uim_id)
            gaupol.framerate_actions[framerate] = action.get_name()
            self.application.framerate_combo.append_text(action.get_label())
        self.application.uim.ensure_update()
        self.application.set_menu_notify_events("custom-framerates")

    def _remove_framerates(self):
        """Remove custom framerates and corresponding UI elements."""
        fallback = aeidon.framerates.FPS_23_976
        combo = self.application.framerate_combo
        store = self.application.framerate_combo.get_model()
        for framerate in reversed(self._framerates):
            if gaupol.conf.editor.framerate == framerate:
                gaupol.conf.editor.framerate = fallback
            if self.application.pages:
                ## Go through all application's pages and reset those set to
                ## the custom framerate back to the default framerate.
                orig_page = self.application.get_current_page()
                for page in self.application.pages:
                    self.application.set_current_page(page)
                    if page.project.framerate == framerate:
                        combo.set_active(fallback)
                self.application.set_current_page(orig_page)
            elif combo.get_active() == framerate:
                ## If no pages are open, but the framerate is set to the custom
                ## one, reset back to the default framerate, but without
                ## triggering callbacks that assume there are pages.
                callback = self.application._on_framerate_combo_changed
                combo.handler_block_by_func(callback)
                combo.set_active(fallback)
                combo.handler_unblock_by_func(callback)
                action = self.application.get_framerate_action(fallback)
                callback = self.application._on_show_framerate_23_976_changed
                action.handler_block_by_func(callback)
                action.set_active(True)
                action.handler_unblock_by_func(callback)
            ## Remove UI elements created for the custom framerate and finally
            ## remove the custom framerate from its enumeration.
            store.remove(store.get_iter(framerate))
            del gaupol.framerate_actions[framerate]
            name = "FPS_%s" % (("%.3f" % framerate.value).replace(".", "_"))
            delattr(aeidon.framerates, name)
        for uim_id in self._uim_ids:
            self.application.uim.remove_ui(uim_id)
        self.application.uim.remove_action_group(self._action_group)
        self.application.uim.ensure_update()

    def setup(self, application):
        """Setup extension for use with `application`."""
        gaupol.conf.register_extension("custom_framerates",
                                       {"framerates": [48.0]})

        self._conf = gaupol.conf.extensions.custom_framerates
        self.application = application
        self._add_framerates()

    def show_preferences_dialog(self, parent):
        """Show a dialog to edit list of custom framerates."""
        dialog = PreferencesDialog(self._conf.framerates, parent)
        gaupol.util.run_dialog(dialog)
        self._conf.framerates = list(dialog.get_framerates())
        dialog.destroy()
        self._remove_framerates()
        self._add_framerates()

    def teardown(self, application):
        """End use of extension with `application`."""
        self._remove_framerates()
        self._clear_attributes()



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon

from .test_microdvd import TestMicroDVD


class TestMPL2(TestMicroDVD):

    text = ("All things weird are normal\n"
            "in this whore of cities.")

    def setup_method(self, method):

        self.markup = aeidon.tags.new(aeidon.formats.MPL2)

    def test_bolden(self):
        assert self.markup.bolden(self.text, (0, 27)) == (
            "\\All things weird are normal\n"
            "in this whore of cities.")

    def test_decode__bold(self):
        text = ("\\All things weird are normal\n"
                "\\in this whore of cities.")
        assert self.markup.decode(text) == (
            "<b>All things weird are normal</b>\n"
            "<b>in this whore of cities.</b>")

    def test_decode__italic(self):
        text = ("/All things weird are normal\n"
                "in this whore of cities.")
        assert self.markup.decode(text) == (
            "<i>All things weird are normal</i>\n"
            "in this whore of cities.")

    def test_decode__multiple(self):
        text = ("/_All things weird are normal\n"
                "\\/_in this whore of cities.")
        assert self.markup.decode(text) == (
            "<i><u>All things weird are normal</u></i>\n"
            "<b><i><u>in this whore of cities.</u></i></b>")

    def test_decode__underline(self):
        text = ("All things weird are normal\n"
                "_in this whore of cities.")
        assert self.markup.decode(text) == (
            "All things weird are normal\n"
            "<u>in this whore of cities.</u>")

    def test_encode__bold(self):
        text = ("<b>All things weird are normal\n"
                "in this whore of cities.</b>")
        assert self.markup.encode(text) == (
            "\\All things weird are normal\n"
            "\\in this whore of cities.")

    def test_encode__italic(self):
        text = ("All <i>things</i> weird are normal\n"
                "in <i>this</i> whore of cities.")
        assert self.markup.encode(text) == (
            "All things weird are normal\n"
            "in this whore of cities.")

    def test_encode__underline(self):
        text = ("All things weird are normal\n"
                "<u>in this whore of cities</u>.")
        assert self.markup.encode(text) == (
            "All things weird are normal\n"
            "_in this whore of cities.")

    def test_italic_tag(self):
        assert self.markup.italic_tag.match("/")
        assert self.markup.italic_tag.match("{Y:i}")
        assert self.markup.italic_tag.match("{y:i}")

    def test_italicize(self):
        assert self.markup.italicize(self.text) == (
            "/All things weird are normal\n"
            "/in this whore of cities.")

    def test_tag(self):
        assert self.markup.tag.match("\\")
        assert self.markup.tag.match("/")
        assert self.markup.tag.match("_")
        assert self.markup.tag.match("{y:ibu}")
        assert self.markup.tag.match("{c:$000000}")

    def test_underline(self):
        assert self.markup.underline(self.text, (0, 3)) == (
            "All things weird are normal\n"
            "in this whore of cities.")



# Copyright (C) 2006-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import os
import sys


class TestModule(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()

    def test_benchmark(self):
        function = aeidon.deco.benchmark(lambda x: x ** 2)
        assert function(2) == 4
        assert function(2) == 4

    @aeidon.deco.monkey_patch(aeidon, "debug")
    def test_contractual__check(self):
        aeidon.debug = True
        reload(aeidon.util)
        aeidon.util.get_ranges((1, 2, 3))

    @aeidon.deco.monkey_patch(aeidon, "debug")
    def test_contractual__skip(self):
        aeidon.debug = False
        reload(aeidon.util)
        aeidon.util.get_ranges((1, 2, 3))

    def test_export(self):
        function = aeidon.deco.export(lambda x: x ** 2)
        assert function.export is True

    def test_memoize(self):
        function = aeidon.deco.memoize(lambda x: x ** 2)
        assert function(2) == 4
        assert function(2) == 4

    def test_monkey_patch__no_attribute(self):
        @aeidon.deco.monkey_patch(sys, "aeidon")
        def modify_aeidon():
            sys.aeidon = True
        modify_aeidon()
        assert not hasattr(sys, "aeidon")

    def test_monkey_patch__os_environ(self):
        @aeidon.deco.monkey_patch(os, "environ")
        def modify_environment():
            os.environ["AEIDON_TEST"] = "1"
        modify_environment()
        assert not "AEIDON_TEST" in os.environ

    def test_monkey_patch__sys_platform(self):
        platform = sys.platform
        @aeidon.deco.monkey_patch(sys, "platform")
        def modify_platform():
            sys.platform = "commodore_64"
        modify_platform()
        assert sys.platform == platform

    def test_notify_frozen(self):
        self.project.insert_subtitles((0,))
        assert not self.project.thaw_notify(True)

    def test_once(self):
        function = aeidon.deco.once(lambda: 5)
        assert function() == 5
        assert function() == 5

    @aeidon.deco.reversion_test
    def test_reversion_test(self):
        self.project.remove_subtitles((0,))

    def test_revertable(self):
        project = self.project
        project.remove_subtitles((0,), register=None)
        assert len(project.undoables) == 0
        assert len(project.redoables) == 0
        project.clear_texts((0,), aeidon.documents.MAIN)
        assert len(project.undoables) == 1
        assert len(project.redoables) == 0
        project.remove_subtitles((0,))
        assert len(project.undoables) == 2
        assert len(project.redoables) == 0

    def test_silent(self):
        function = lambda: [].remove(None)
        aeidon.deco.silent(ValueError)(function)()
        aeidon.deco.silent(Exception)(function)()
        aeidon.deco.silent()(function)()



# Copyright (C) 2011 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk
import os
import sys
import traceback


class TestAddFramerateDialog(gaupol.TestCase):

    def run__dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        directory = os.path.abspath(os.path.dirname(__file__))
        directory = os.path.abspath(os.path.join(directory, ".."))
        sys.path.insert(0, directory)
        try: mobj = __import__("custom-framerates", {}, {}, [])
        except ImportError:
            return traceback.print_exc()
        finally: sys.path.pop(0)
        self.dialog = mobj.AddFramerateDialog(gtk.Window())
        self.dialog.show()

    def test__on_response__cancel(self):
        self.dialog.response(gtk.RESPONSE_CANCEL)

    def test__on_response__ok(self):
        self.dialog.response(gtk.RESPONSE_OK)

    def test_get_framerate(self):
        assert self.dialog.get_framerate() == 0.0


class TestPreferencesDialog(gaupol.TestCase):

    def run__dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        directory = os.path.abspath(os.path.dirname(__file__))
        directory = os.path.abspath(os.path.join(directory, ".."))
        sys.path.insert(0, directory)
        try: mobj = __import__("custom-framerates", {}, {}, [])
        except ImportError:
            return traceback.print_exc()
        finally: sys.path.pop(0)
        self.framerates = (20.0, 21.0, 22.0, 23.0)
        self.dialog = mobj.PreferencesDialog(self.framerates, gtk.Window())
        self.dialog.show()

    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_add_button_clicked(self):
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_OK
        orig_framerates = self.dialog.get_framerates()
        self.dialog._add_button.emit("clicked")
        framerates = self.dialog.get_framerates()
        assert len(framerates) == len(orig_framerates) + 1
        assert 0.0 in framerates

    def test__on_remove_button_clicked(self):
        orig_framerates = self.dialog.get_framerates()
        selection = self.dialog._tree_view.get_selection()
        selection.unselect_all()
        selection.select_path(0)
        self.dialog._remove_button.emit("clicked")
        framerates = self.dialog.get_framerates()
        assert len(framerates) == len(orig_framerates) - 1

    def test__on_tree_view_selection_changed(self):
        selection = self.dialog._tree_view.get_selection()
        selection.unselect_all()
        store = self.dialog._tree_view.get_model()
        for i in range(len(store)):
            selection.select_path(i)

    def test_get_framerates(self):
        framerates = self.dialog.get_framerates()
        assert framerates == self.framerates


class TestCustomFrameratesExtension(gaupol.TestCase):

    def setup_method(self, method):
        directory = os.path.abspath(os.path.dirname(__file__))
        directory = os.path.abspath(os.path.join(directory, ".."))
        sys.path.insert(0, directory)
        try: mobj = __import__("custom-framerates", {}, {}, [])
        except ImportError:
            return traceback.print_exc()
        finally: sys.path.pop(0)
        self.extension = mobj.CustomFrameratesExtension()
        self.application = self.new_application()
        self.extension.setup(self.application)

    def teardown_method(self, method):
        self.extension.teardown(self.application)
        gaupol.TestCase.teardown_method(self, self.application)

    def test_setup(self):
        assert hasattr(aeidon.framerates, "FPS_48_000")

    def test_setup__conf(self):
        assert hasattr(aeidon.framerates, "FPS_48_000")

    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test_show_preferences_dialog(self):
        gaupol.util.run_dialog = lambda *args: gtk.RESPONSE_CLOSE
        self.extension.show_preferences_dialog(self.application.window)

    def test_teardown(self):
        self.extension.teardown(self.application)
        assert not hasattr(aeidon.framerates, "FPS_48_000")
        self.extension.setup(self.application)

    def test_teardown__custom(self):
        # pylint: disable=E1101
        page = self.application.get_current_page()
        page.project.set_framerate(aeidon.framerates.FPS_48_000)
        self.extension.teardown(self.application)
        self.extension.setup(self.application)

    def test_teardown__custom_no_pages(self):
        # pylint: disable=E1101
        page = self.application.get_current_page()
        page.project.set_framerate(aeidon.framerates.FPS_48_000)
        while self.application.pages:
            page = self.application.get_current_page()
            self.application.close(page, confirm=False)
        self.extension.teardown(self.application)
        self.extension.setup(self.application)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import os


class TestPreviewAgent(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()
        self.delegate = self.project.preview.im_self

    def test_find_video__avi(self):
        video_path = aeidon.temp.create(".avi")
        sub_path = video_path.replace(".avi", ".srt")
        sfile = aeidon.files.new(aeidon.formats.SUBRIP,
                                 sub_path,
                                 "ascii")

        self.project.save_main(sfile)
        self.project.find_video()
        assert self.project.video_path == video_path
        os.remove(sub_path)

    def test_find_video__none(self):
        self.project.find_video()
        assert self.project.video_path is None

    def test_preview__encoding(self):
        doc = aeidon.documents.MAIN
        self.project.video_path = self.new_subrip_file()
        value = self.project.preview("00:00:00.000", doc, "echo", 0, "utf_8")
        assert os.path.isfile(self.project.get_file(doc).path)

    def test_preview__main(self):
        doc = aeidon.documents.MAIN
        self.project.video_path = self.new_subrip_file()
        value = self.project.preview("00:00:00.000", doc, "echo", 0)
        assert os.path.isfile(self.project.get_file(doc).path)

    def test_preview__translation(self):
        doc = aeidon.documents.TRAN
        self.project.video_path = self.new_subrip_file()
        value = self.project.preview("00:00:00.000", doc, "echo", 0)
        assert os.path.isfile(self.project.get_file(doc).path)



# Copyright (C) 2006-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol


class TestModule(gaupol.TestCase):

    def test__check_dependencies(self):
        gaupol.main._check_dependencies()

    def test__init_application(self):
        opts = type("", (object,), {})
        opts.encoding = "ascii"
        opts.translation_file = self.new_subrip_file()
        opts.align_method = "position"
        opts.video_file = self.new_subrip_file()
        args = [self.new_subrip_file(), "+3"]
        gaupol.main._init_application(opts, args)

    def test__init_configuration(self):
        path = self.new_subrip_file()
        open(path, "w").write("\n")
        gaupol.main._init_configuration(path)

    def test__on_parser_list_encodings(self):
        self.raises(SystemExit, gaupol.main._on_parser_list_encodings)

    def test__on_parser_version(self):
        self.raises(SystemExit, gaupol.main._on_parser_version)

    def test__parse_args(self):
        gaupol.main._parse_args([])



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk


class TestMultiCloseDialog(gaupol.TestCase):

    def run__dialog__both(self):
        self.setup_both()
        self.dialog.run()
        self.dialog.destroy()

    def run__dialog__main(self):
        self.setup_main()
        self.dialog.run()
        self.dialog.destroy()

    def run__dialog__translation(self):
        self.setup_translation()
        self.dialog.run()
        self.dialog.destroy()

    def setup_both(self):
        # pylint: disable=W0201
        self.application = self.new_application()
        for page in self.application.pages:
            page.project.remove_subtitles((0,))
        self.dialog = gaupol.MultiCloseDialog(self.application.window,
                                              self.application,
                                              self.application.pages)

        self.dialog.show()

    def setup_main(self):
        # pylint: disable=W0201
        self.application = self.new_application()
        for page in self.application.pages:
            page.project.set_text(0, aeidon.documents.MAIN, "")
        self.dialog = gaupol.MultiCloseDialog(self.application.window,
                                              self.application,
                                              self.application.pages)

        self.dialog.show()

    def setup_method(self, method):
        return self.setup_both()

    def setup_translation(self):
        # pylint: disable=W0201
        self.application = self.new_application()
        for page in self.application.pages:
            page.project.set_text(0, aeidon.documents.TRAN, "")
        self.dialog = gaupol.MultiCloseDialog(self.application.window,
                                              self.application,
                                              self.application.pages)

        self.dialog.show()

    def test___init__(self):
        assert self.dialog.pages is not self.application.pages

    def test__on_response__no(self):
        self.dialog.response(gtk.RESPONSE_NO)

    def test__on_response__yes(self):
        self.dialog.response(gtk.RESPONSE_YES)

    def test__on_tree_view_cell_toggled__main(self):
        column = self.dialog._main_tree_view.get_columns()[0]
        renderer = column.get_cell_renderers()[0]
        renderer.emit("toggled", 0)
        renderer.emit("toggled", 0)

    def test__on_tree_view_cell_toggled__translation(self):
        column = self.dialog._tran_tree_view.get_columns()[0]
        renderer = column.get_cell_renderers()[0]
        renderer.emit("toggled", 0)
        renderer.emit("toggled", 0)



# Copyright (C) 2007-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import os


class TestModule(aeidon.TestCase):

    def test_close(self):
        path = aeidon.temp.create()
        aeidon.temp.close(path)
        aeidon.temp.remove(path)

    def test_create(self):
        path = aeidon.temp.create()
        aeidon.temp.remove(path)

    def test_create_directory(self):
        path = aeidon.temp.create_directory()
        aeidon.temp.remove_directory(path)

    def test_get_handle(self):
        path = aeidon.temp.create()
        aeidon.temp.get_handle(path)
        aeidon.temp.remove(path)

    def test_remove(self):
        path = aeidon.temp.create()
        aeidon.temp.remove(path)
        assert not os.path.isfile(path)
        aeidon.temp.remove(path)

    def test_remove_all(self):
        path_1 = aeidon.temp.create()
        path_2 = aeidon.temp.create()
        aeidon.temp.remove_all()
        assert not os.path.isfile(path_1)
        assert not os.path.isfile(path_2)

    def test_remove_directory(self):
        path = aeidon.temp.create_directory()
        open(os.path.join(path, "a"), "w").write("a")
        open(os.path.join(path, "b"), "w").write("b")
        os.makedirs(os.path.join(path, "c"))
        aeidon.temp.remove_directory(path)
        assert not os.path.isdir(path)
        aeidon.temp.remove_directory(path)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Text parser for markup-tag-aware text editing."""

import aeidon

__all__ = ("Parser",)


class Parser(aeidon.Finder):

    """Text parser for markup-tag-aware text editing.

    :ivar _margins: Start tag, end tag that every line is wrapped in
    :ivar _tags: List of lists of markup tag, position
    :ivar clean_func: Function to clean tags or ``None``
    :ivar re_tag: Regular expression object to match any tag

    The purpose of the :class:`Parser` is to split text to the actual text and
    its markup tags, allowing the text to be edited while keeping the tags
    separate and intact. An example would be replacing all "i"s with "j"s
    without changing italic markup::

        >>> parser = aeidon.Parser(re.compile(r"<.+?>"))
        >>> parser.set_text("<i>iii</i>")
        >>> parser.pattern = "i"
        >>> parser.replacement = "j"
        >>> parser.replace_all()
        3
        >>> parser.get_text()
        '<i>jjj</i>'

    The margin system (wrapping each line in the same set of tags) is only used
    if no other tags are found in the text and if the text has at least two
    lines. Either margins or tags will always be empty.
    """

    __metaclass__ = aeidon.Contractual

    def __init__(self, re_tag=None, clean_func=None):
        """Initialize a :class:`Parser` object."""
        aeidon.Finder.__init__(self)
        self._margins = None
        self._tags = None
        self.clean_func = clean_func
        self.re_tag = re_tag

    def _invariant(self):
        if self.clean_func is not None:
            assert self.clean_func("") == ""

    def _set_margins_require(self, text):
        assert self.re_tag is not None

    def _set_margins(self, text):
        """Find the margin markup tags in `text` if such exist."""
        lines = text.split("\n")
        line = lines[0]
        start_tag = ""
        while True:
            match = self.re_tag.match(line)
            if match is None: break
            a, z = match.span()
            start_tag += line[a:z]
            line = line[z:]
        if not start_tag: return
        if not all([x.startswith(start_tag) for x in lines]): return

        end_tag = ""
        while True:
            iterator = self.re_tag.finditer(line)
            match = aeidon.util.last(iterator)
            if match is None: break
            a, z = match.span()
            if z != len(line): return
            end_tag = line[a:z] + end_tag
            line = line[:a]
        if not all([x.endswith(end_tag) for x in lines]): return

        for line in (x[len(start_tag):-len(end_tag)] for x in lines):
            # Ensure that no other tags exists on any of the lines.
            if self.re_tag.search(line) is not None: return

        self._margins = [start_tag, end_tag]

    def _set_tags_require(self, text):
        assert self.re_tag is not None

    def _set_tags(self, text):
        """Find markup tags in `text`."""
        for match in self.re_tag.finditer(text):
            a, z = match.span()
            self._tags.append([a, text[a:z]])

    def _shift_tags(self, pos, shift, orig_text):
        """Shift all markup tags after `pos`."""

        if not shift: return
        if not self._tags: return

        # Try to determine whether a tag at position pos would be an opening
        # or a closing tag, i.e. attached to the next or the previous word.
        opening = True
        if pos < len(orig_text):
            if orig_text[pos].isspace():
                opening = False
        elif pos == len(orig_text):
            opening = False
        closing = not opening

        # Get length of tags *before* position. Try to add strings (positive
        # shift) inside tags and remove strings (negative shift) after tags.
        pos_with_tags = pos
        for tag_pos, tag in self._tags:
            if (shift > 0) and closing:
                if tag_pos < pos_with_tags:
                    pos_with_tags += len(tag)
            elif tag_pos <= pos_with_tags:
                pos_with_tags += len(tag)

        between_length = 0
        for i, (tag_pos, tag) in enumerate(self._tags):
            orig_end = pos_with_tags - shift + between_length
            if (shift < 0) and (pos_with_tags < tag_pos < orig_end):
                # If tag is in the middle of what is being removed, it must be
                # shifted to the start of the removal block, but not so far
                # that it would overlap with preceding tags.
                self._tags[i][0] = pos_with_tags + between_length
                between_length += len(tag)
            elif tag_pos >= pos_with_tags:
                self._tags[i][0] += shift

    def get_text(self):
        """Reassemble the full text and return it."""
        if not self.text:
            self._margins = []
            self._tags = []
        text = self.text[:]
        for pos, tag in self._tags:
            text = text[:pos] + tag + text[pos:]
        if self._margins:
            text = text.replace("\n", "%s\n%s" % tuple(self._margins[::-1]))
            text = self._margins[0] + text + self._margins[1]
        if self.clean_func is not None:
            text = self.clean_func(text)
        return text

    def replace(self, next=True):
        """Replace the current match of pattern.

        `next` should be ``True`` to finish at end of match, ``False`` for
        beginning. Raise :exc:`re.error` if bad replacement.
        """
        a = self.match_span[0]
        orig_text = self.text[:]
        aeidon.Finder.replace(self, next)
        shift = len(self.text) - len(orig_text)
        self._shift_tags(a, shift, orig_text)

    def set_text(self, text, next=True):
        """Set the target text to search in and parse it.

        `next` should be ``True`` to start at beginning, ``False`` for end.
        """
        aeidon.Finder.set_text(self, text, next)
        self._margins = []
        self._tags = []
        if self.re_tag is None: return
        if text.count("\n"):
            self._set_margins(text)
        if not self._margins:
            self._set_tags(text)
        self.text = self.re_tag.sub("", text)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol


class TestUtilityAgent(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()

    def test_get_action(self):
        self.application.get_action("open_main_files")
        self.application.get_action("show_times")
        self.application.get_action("toggle_number_column")

    def test_get_action(self):
        self.raises(ValueError, self.application.get_action, "xxx")

    def test_get_action_group(self):
        self.application.get_action_group("main-safe")
        self.application.get_action_group("main-unsafe")
        self.application.get_action_group("projects")

    def test_get_column_action(self):
        for field in gaupol.fields:
            self.application.get_column_action(field)

    def test_get_current_page(self):
        page = self.application.pages[0]
        self.application.set_current_page(page)
        value = self.application.get_current_page()
        assert value == page

    def test_get_framerate_action(self):
        for framerate in aeidon.framerates:
            self.application.get_framerate_action(framerate)

    def test_get_menu_item(self):
        self.application.get_menu_item("open_main_files")
        self.application.get_menu_item("show_times")
        self.application.get_menu_item("toggle_number_column")

    def test_get_mode_action(self):
        for mode in aeidon.modes:
            self.application.get_mode_action(mode)

    def test_get_target_pages(self):
        get_pages = self.application.get_target_pages
        pages = get_pages(gaupol.targets.SELECTED)
        assert pages == (self.application.get_current_page(),)
        pages = get_pages(gaupol.targets.CURRENT)
        assert pages == (self.application.get_current_page(),)
        pages = get_pages(gaupol.targets.ALL)
        assert pages == tuple(self.application.pages)

    def test_get_target_rows(self):
        page = self.application.get_current_page()
        page.view.select_rows((1, 2, 3))
        get_rows = self.application.get_target_rows
        assert get_rows(gaupol.targets.SELECTED) == (1, 2, 3)
        assert get_rows(gaupol.targets.CURRENT) is None
        assert get_rows(gaupol.targets.ALL) is None

    def test_get_tool_item(self):
        self.application.get_tool_item("undo_action")
        self.application.get_tool_item("insert_subtitles")

    def test_set_current_page(self):
        for i, page in enumerate(self.application.pages):
            self.application.set_current_page(page)
            value = self.application.get_current_page()
            assert value == page



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestClipboard(aeidon.TestCase):

    def setup_method(self, method):
        self.clipboard = aeidon.Clipboard()

    def test_append(self):
        self.clipboard.set_texts(("", None))
        texts = self.clipboard.get_texts()
        assert texts == ["", None]

    def test_clear(self):
        self.clipboard.set_texts(("", None))
        self.clipboard.clear()
        texts = self.clipboard.get_texts()
        assert texts == []

    def test_get_string(self):
        self.clipboard.set_texts(("test", None, "test", None))
        string = self.clipboard.get_string()
        assert string == "test\n\n\n\ntest\n\n"

    def test_get_texts(self):
        self.clipboard.set_texts(("", None))
        texts = self.clipboard.get_texts()
        assert texts == ["", None]

    def test_set_texts(self):
        self.clipboard.set_texts(("", None))
        texts = self.clipboard.get_texts()
        assert texts == ["", None]

    def test_is_empty(self):
        assert self.clipboard.is_empty()
        self.clipboard.append("")
        assert not self.clipboard.is_empty()
